@@ -20,13 +20,21 @@
  * limitations under the License.
  */
 
+#include <cairo.h>
 #include <libbase/libbase.h>
 #include <libwlclient/libwlclient.h>
+#include <math.h>
 #include <primitives/primitives.h>
 #include <primitives/segment_display.h>
-
-#include <math.h>
+#include <stdbool.h>
+#include <stdint.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 #include <sys/time.h>
+#include <time.h>
+
+#include "libwlclient/icon.h"
 
 /** Foreground color of a LED in the VFD-style display. */
 static const uint32_t color_led = 0xff55ffff;
@@ -46,12 +54,10 @@ uint64_t next_draw_time(void)
 /**
  * Draws contents into the icon buffer.
  *
- * @param icon_ptr
  * @param gfxbuf_ptr
  * @param ud_ptr
  */
 bool icon_callback(
-    __UNUSED__ wlclient_icon_t *icon_ptr,
     bs_gfxbuf_t *gfxbuf_ptr,
     __UNUSED__ void *ud_ptr)
 {
@@ -193,7 +199,7 @@ void timer_callback(wlclient_t *client_ptr, void *ud_ptr)
 {
     wlclient_icon_t *icon_ptr = ud_ptr;
 
-    wlclient_icon_callback_when_ready(icon_ptr, icon_callback, NULL);
+    wlclient_icon_register_ready_callback(icon_ptr, icon_callback, NULL);
     wlclient_register_timer(
         client_ptr, next_draw_time(), timer_callback, icon_ptr);
 }
@@ -204,7 +210,7 @@ int main(__UNUSED__ int argc, __UNUSED__ char **argv)
 {
     bs_log_severity = BS_DEBUG;
 
-    wlclient_t *wlclient_ptr = wlclient_create("wlmclock");
+    wlclient_t *wlclient_ptr = wlclient_create("wlmaker.wlmeyes");
     if (NULL == wlclient_ptr) return EXIT_FAILURE;
 
     if (wlclient_icon_supported(wlclient_ptr)) {
@@ -212,7 +218,8 @@ int main(__UNUSED__ int argc, __UNUSED__ char **argv)
         if (NULL == icon_ptr) {
             bs_log(BS_ERROR, "Failed wlclient_icon_create(%p)", wlclient_ptr);
         } else {
-            wlclient_icon_callback_when_ready(icon_ptr, icon_callback, NULL);
+            wlclient_icon_register_ready_callback(
+                icon_ptr, icon_callback, NULL);
 
             wlclient_register_timer(
                 wlclient_ptr, next_draw_time(), timer_callback, icon_ptr);
