@@ -60,8 +60,8 @@ wlmaker_dock_t *wlmaker_dock_create(
  */
 void wlmaker_dock_destroy(wlmaker_dock_t *dock_ptr);
 
-/** Unit test cases. */
-extern const bs_test_case_t wlmaker_dock_test_cases[];
+/** Unit test test. */
+extern const bs_test_set_t wlmaker_dock_test_set;
 
 #ifdef __cplusplus
 }  // extern "C"
