@@ -20,14 +20,22 @@
 
 #include "icon_manager.h"
 
+#include <inttypes.h>
 #include <libbase/libbase.h>
-
+#include <stdbool.h>
+#include <stdlib.h>
+#include <wayland-server-core.h>
 #define WLR_USE_UNSTABLE
+#include <wlr/types/wlr_compositor.h>
 #include <wlr/types/wlr_xdg_shell.h>
 #undef WLR_USE_UNSTABLE
 
 #include "toolkit/toolkit.h"
 #include "wlmaker-icon-unstable-v1-server-protocol.h"
+#include "xdg_shell.h"
+
+struct wl_client;
+struct wl_resource;
 
 /* == Declarations ========================================================= */
 
@@ -360,8 +368,7 @@ wlmaker_toplevel_icon_t *wlmaker_toplevel_icon_create(
 
     if (!wlmtk_tile_init(
             &toplevel_icon_ptr->super_tile,
-            &icon_manager_ptr->server_ptr->style.tile,
-            icon_manager_ptr->server_ptr->env_ptr)) {
+            &icon_manager_ptr->server_ptr->style_ptr->tile)) {
         wlmaker_toplevel_icon_destroy(toplevel_icon_ptr);
         return NULL;
     }
@@ -377,7 +384,7 @@ wlmaker_toplevel_icon_t *wlmaker_toplevel_icon_create(
 
     toplevel_icon_ptr->content_surface_ptr = wlmtk_surface_create(
         wlr_surface_ptr,
-        icon_manager_ptr->server_ptr->env_ptr);
+        icon_manager_ptr->server_ptr->wlr_seat_ptr);
     if (NULL == toplevel_icon_ptr->content_surface_ptr) {
         wlmaker_toplevel_icon_destroy(toplevel_icon_ptr);
         return NULL;
