@@ -28,7 +28,9 @@
 #include <xkbcommon/xkbcommon.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/backend.h>
+#include <wlr/types/wlr_data_control_v1.h>
 #include <wlr/types/wlr_data_device.h>
+#include <wlr/types/wlr_primary_selection_v1.h>
 #include <wlr/types/wlr_seat.h>
 #undef WLR_USE_UNSTABLE
 
@@ -115,11 +117,24 @@ struct _wlmaker_server_t {
     /** Listener for `new_input` signals raised by `wlr_backend`. */
     struct wl_listener        backend_new_input_device_listener;
 
-    // From tinywl.c: A few hands-off wlroots interfaces.
+    // Clipboard and selection support.
 
-    /** The data device manager handles the clipboard. */
+    /** Provides the wl_data_device protocol for clipboard (Ctrl+C/V). */
     struct wlr_data_device_manager *wlr_data_device_manager_ptr;
 
+    /** Listener to approve clipboard selection requests. */
+    struct wl_listener        request_set_selection_listener;
+
+    /** Provides the primary selection protocol for middle-click paste. */
+    struct wlr_primary_selection_v1_device_manager
+        *wlr_primary_selection_v1_device_manager_ptr;
+
+    /** Listener to approve primary selection requests. */
+    struct wl_listener        request_set_primary_selection_listener;
+
+    /** Enables clipboard managers and tools (wl-copy, wl-paste). */
+    struct wlr_data_control_manager_v1 *wlr_data_control_manager_v1_ptr;
+
     /** The cursor handler. */
     wlmaker_cursor_t          *cursor_ptr;
     /** The XDG Shell handler. */
@@ -183,7 +198,7 @@ struct _wlmaker_server_t {
     struct wl_listener        unclaimed_button_event_listener;
 
     /** The current configuration style. */
-    wlmaker_config_style_t    style;
+    const wlmaker_config_style_t *style_ptr;
 };
 
 /** Specifies the key + modifier to bind. */
@@ -203,7 +218,8 @@ struct _wlmaker_key_combo_t {
  *
  * @param config_dict_ptr     Configuration, as dictionary object. The server
  *                            will keep a reference on it until destroyed.
- * @param files_ptr
+ * @param files_ptr           Files handle. Must outlive the server.
+ * @param style_ptr           Style. Must outlive the server.
  * @param options_ptr         Options for the server. The server expects the
  *                            pointed area to outlive the server.
  *
@@ -213,6 +229,7 @@ struct _wlmaker_key_combo_t {
 wlmaker_server_t *wlmaker_server_create(
     bspl_dict_t *config_dict_ptr,
     wlmaker_files_t *files_ptr,
+    const wlmaker_config_style_t *style_ptr,
     const wlmaker_server_options_t *options_ptr);
 
 /**
