@@ -18,46 +18,39 @@
  * limitations under the License.
  */
 
-#include "element.h"
 #include "container.h"
-
+#include "element.h"
+#include "input.h"
 #include "util.h"
 
+#include <math.h>
+#include <stdlib.h>
+#include <wayland-util.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/types/wlr_keyboard.h>
 #include <wlr/types/wlr_scene.h>
 #undef WLR_USE_UNSTABLE
+#include <wlr/util/box.h>
+#include <libbase/libbase.h>
 
 /* == Declarations ========================================================= */
 
-static void _wlmtk_element_get_pointer_area(
-    wlmtk_element_t *element_ptr,
-    int *x1_ptr,
-    int *y1_ptr,
-    int *x2_ptr,
-    int *y2_ptr);
-static bool _wlmtk_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x,
-    double y,
-    uint32_t time_msec);
 static bool _wlmtk_element_pointer_button(
     wlmtk_element_t *element_ptr,
     const wlmtk_button_event_t *button_event_ptr);
 static bool _wlmtk_element_pointer_axis(
     __UNUSED__ wlmtk_element_t *element_ptr,
     __UNUSED__ struct wlr_pointer_axis_event *wlr_pointer_axis_event_ptr);
-static void _wlmtk_element_pointer_enter(
-    wlmtk_element_t *element_ptr);
-static void _wlmtk_element_pointer_leave(
-    wlmtk_element_t *element_ptr);
 static void _wlmtk_element_keyboard_blur(wlmtk_element_t *element_ptr);
 static bool _wlmtk_element_keyboard_event(
     wlmtk_element_t *element_ptr,
-    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr,
-    const xkb_keysym_t *key_syms,
-    size_t key_syms_count,
+    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr);
+static bool _wlmtk_element_keyboard_sym(
+    wlmtk_element_t *element_ptr,
+    xkb_keysym_t keysym,
+    enum xkb_key_direction direction,
     uint32_t modifiers);
+static void _wlmtk_element_layout(wlmtk_element_t *element_ptr);
 
 static void handle_wlr_scene_node_destroy(
     struct wl_listener *listener_ptr,
@@ -67,31 +60,28 @@ static void handle_wlr_scene_node_destroy(
 
 /** Default virtual method table. Initializes the non-abstract methods. */
 static const wlmtk_element_vmt_t element_vmt = {
-    .get_pointer_area = _wlmtk_element_get_pointer_area,
-    .pointer_motion = _wlmtk_element_pointer_motion,
     .pointer_button = _wlmtk_element_pointer_button,
     .pointer_axis = _wlmtk_element_pointer_axis,
-    .pointer_enter = _wlmtk_element_pointer_enter,
-    .pointer_leave = _wlmtk_element_pointer_leave,
     .keyboard_blur = _wlmtk_element_keyboard_blur,
     .keyboard_event = _wlmtk_element_keyboard_event,
+    .keyboard_sym = _wlmtk_element_keyboard_sym,
+    .layout = _wlmtk_element_layout,
 };
 
 /* == Exported methods ===================================================== */
 
 /* ------------------------------------------------------------------------- */
-bool wlmtk_element_init(
-    wlmtk_element_t *element_ptr,
-    wlmtk_env_t *env_ptr)
+bool wlmtk_element_init(wlmtk_element_t *element_ptr)
 {
     BS_ASSERT(NULL != element_ptr);
-    memset(element_ptr, 0, sizeof(wlmtk_element_t));
-    element_ptr->vmt = element_vmt;
-    element_ptr->env_ptr = env_ptr;
+    *element_ptr = (wlmtk_element_t){ .vmt = element_vmt };
 
-    element_ptr->last_pointer_x = NAN;
-    element_ptr->last_pointer_y = NAN;
-    element_ptr->last_pointer_time_msec = 0;
+    wl_signal_init(&element_ptr->events.pointer_enter);
+    wl_signal_init(&element_ptr->events.pointer_leave);
+    wl_signal_init(&element_ptr->events.pointer_motion);
+
+    element_ptr->last_pointer_motion_event = (wlmtk_pointer_motion_event_t){
+        .x = NAN, .y = NAN, .time_msec = 0 };
     return true;
 }
 
@@ -112,11 +102,9 @@ wlmtk_element_vmt_t wlmtk_element_extend(
     if (NULL != element_vmt_ptr->get_dimensions) {
         element_ptr->vmt.get_dimensions = element_vmt_ptr->get_dimensions;
     }
-    if (NULL != element_vmt_ptr->get_pointer_area) {
-        element_ptr->vmt.get_pointer_area = element_vmt_ptr->get_pointer_area;
-    }
-    if (NULL != element_vmt_ptr->pointer_motion) {
-        element_ptr->vmt.pointer_motion = element_vmt_ptr->pointer_motion;
+    if (NULL != element_vmt_ptr->pointer_accepts_motion) {
+        element_ptr->vmt.pointer_accepts_motion =
+            element_vmt_ptr->pointer_accepts_motion;
     }
     if (NULL != element_vmt_ptr->pointer_button) {
         element_ptr->vmt.pointer_button = element_vmt_ptr->pointer_button;
@@ -124,12 +112,6 @@ wlmtk_element_vmt_t wlmtk_element_extend(
     if (NULL != element_vmt_ptr->pointer_axis) {
         element_ptr->vmt.pointer_axis = element_vmt_ptr->pointer_axis;
     }
-    if (NULL != element_vmt_ptr->pointer_enter) {
-        element_ptr->vmt.pointer_enter = element_vmt_ptr->pointer_enter;
-    }
-    if (NULL != element_vmt_ptr->pointer_leave) {
-        element_ptr->vmt.pointer_leave = element_vmt_ptr->pointer_leave;
-    }
     if (NULL != element_vmt_ptr->pointer_grab_cancel) {
         element_ptr->vmt.pointer_grab_cancel =
             element_vmt_ptr->pointer_grab_cancel;
@@ -140,6 +122,12 @@ wlmtk_element_vmt_t wlmtk_element_extend(
     if (NULL != element_vmt_ptr->keyboard_event) {
         element_ptr->vmt.keyboard_event = element_vmt_ptr->keyboard_event;
     }
+    if (NULL != element_vmt_ptr->keyboard_sym) {
+        element_ptr->vmt.keyboard_sym = element_vmt_ptr->keyboard_sym;
+    }
+    if (NULL != element_vmt_ptr->layout) {
+        element_ptr->vmt.layout = element_vmt_ptr->layout;
+    }
 
     return orig_vmt;
 }
@@ -152,7 +140,7 @@ void wlmtk_element_fini(
     BS_ASSERT(NULL == element_ptr->wlr_scene_node_ptr);
     BS_ASSERT(NULL == element_ptr->parent_container_ptr);
 
-    memset(element_ptr, 0, sizeof(wlmtk_element_t));
+    *element_ptr = (wlmtk_element_t){};
 }
 
 /* ------------------------------------------------------------------------- */
@@ -235,7 +223,7 @@ void wlmtk_element_set_visible(wlmtk_element_t *element_ptr, bool visible)
     }
 
     if (NULL != element_ptr->parent_container_ptr) {
-        wlmtk_container_update_layout(element_ptr->parent_container_ptr);
+        wlmtk_container_invalidate_layout(element_ptr->parent_container_ptr);
     }
 }
 
@@ -271,66 +259,105 @@ void wlmtk_element_set_position(
     element_ptr->y = y;
 
     if (NULL != element_ptr->parent_container_ptr) {
-        wlmtk_container_update_pointer_focus(
-            element_ptr->parent_container_ptr);
+        wlmtk_container_invalidate_layout(element_ptr->parent_container_ptr);
     }
 }
 
 /* ------------------------------------------------------------------------- */
 bool wlmtk_element_pointer_motion(
     wlmtk_element_t *element_ptr,
-    double x,
-    double y,
-    uint32_t time_msec)
+    wlmtk_pointer_motion_event_t *motion_event_ptr)
 {
-    bool within = element_ptr->vmt.pointer_motion(element_ptr, x, y, time_msec);
-    if (within == element_ptr->pointer_inside) return within;
+    element_ptr->last_pointer_motion_event = *motion_event_ptr;
 
-    if (within) {
-        element_ptr->pointer_inside = true;
-        element_ptr->vmt.pointer_enter(element_ptr);
-    } else {
-        element_ptr->pointer_inside = false;
-        element_ptr->vmt.pointer_leave(element_ptr);
+    if (NULL == element_ptr->vmt.pointer_accepts_motion) return false;
+    if (!element_ptr->visible) return false;
+
+    bool inside = BS_ASSERT_NOTNULL(element_ptr->vmt.pointer_accepts_motion)(
+        element_ptr, motion_event_ptr);
+
+    if (inside) {
+        if (wlmtk_element_pointer_focus(element_ptr, motion_event_ptr)) {
+            element_ptr->last_pointer_motion_event = *motion_event_ptr;
+            wl_signal_emit(&element_ptr->events.pointer_motion,
+                           motion_event_ptr);
+            return true;
+        }
     }
 
-    return within;
+    if (element_ptr->pointer_inside) {
+        wlmtk_element_pointer_blur(element_ptr);
+    }
+    return false;
+}
+
+/* ------------------------------------------------------------------------- */
+bool wlmtk_element_pointer_button(
+    wlmtk_element_t *element_ptr,
+    const wlmtk_button_event_t *button_event_ptr)
+{
+    return element_ptr->vmt.pointer_button(element_ptr, button_event_ptr);
+}
+
+/* ------------------------------------------------------------------------- */
+bool wlmtk_element_pointer_axis(
+    wlmtk_element_t *element_ptr,
+    struct wlr_pointer_axis_event *wlr_pointer_axis_event_ptr)
+{
+    return element_ptr->vmt.pointer_axis(
+        element_ptr,
+        wlr_pointer_axis_event_ptr);
+}
+
+/* ------------------------------------------------------------------------- */
+bool wlmtk_element_pointer_focus(
+    wlmtk_element_t *element_ptr,
+    wlmtk_pointer_motion_event_t *motion_event_ptr)
+{
+    if (element_ptr->pointer_inside) return true;
+
+    if (NULL != element_ptr->parent_container_ptr &&
+        !wlmtk_container_request_pointer_focus(
+            element_ptr->parent_container_ptr,
+            element_ptr,
+            motion_event_ptr)) {
+        return false;
+    }
+
+    element_ptr->last_pointer_motion_event = *motion_event_ptr;
+    element_ptr->pointer_inside = true;
+    wl_signal_emit(
+        &element_ptr->events.pointer_enter,
+        motion_event_ptr->pointer_ptr);
+    return true;
+}
+
+/* ------------------------------------------------------------------------- */
+void wlmtk_element_pointer_blur(wlmtk_element_t *element_ptr)
+{
+    if (!element_ptr->pointer_inside ||
+        element_ptr->inhibit_pointer_blur) return;
+
+    element_ptr->pointer_inside = false;
+    wl_signal_emit(&element_ptr->events.pointer_leave, NULL);
+
+    if (NULL != element_ptr->parent_container_ptr) {
+        wlmtk_element_pointer_blur(
+            &element_ptr->parent_container_ptr->super_element);
+    }
+}
+
+/* ------------------------------------------------------------------------- */
+void wlmtk_element_pointer_grab_cancel(
+    wlmtk_element_t *element_ptr)
+{
+    if (NULL != element_ptr->vmt.pointer_grab_cancel) {
+        element_ptr->vmt.pointer_grab_cancel(element_ptr);
+    }
 }
 
 /* == Local (static) methods =============================================== */
 
-/* ------------------------------------------------------------------------- */
-/** Wraps to wlmtk_element_vmt_t::get_dimensions as default implementation. */
-void _wlmtk_element_get_pointer_area(
-    wlmtk_element_t *element_ptr,
-    int *x1_ptr,
-    int *y1_ptr,
-    int *x2_ptr,
-    int *y2_ptr)
-{
-    wlmtk_element_get_dimensions(element_ptr, x1_ptr, y1_ptr, x2_ptr, y2_ptr);
-}
-
-/* ------------------------------------------------------------------------- */
-/** Stores pointer coordinates and timestamp. Returns true is x,y not NAN. */
-bool _wlmtk_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x,
-    double y,
-    uint32_t time_msec)
-{
-    if (isnan(x) || isnan(y)) {
-        x = NAN;
-        y = NAN;
-    }
-
-    element_ptr->last_pointer_x = x;
-    element_ptr->last_pointer_y = y;
-    element_ptr->last_pointer_time_msec = time_msec;
-
-    return !isnan(x) && !isnan(y);
-}
-
 /* ------------------------------------------------------------------------- */
 /** Does nothing, returns false. */
 bool _wlmtk_element_pointer_button(
@@ -349,23 +376,6 @@ bool _wlmtk_element_pointer_axis(
     return false;
 }
 
-/* ------------------------------------------------------------------------- */
-/** Handler for when the pointer enters the area. Sets default cursor. */
-void _wlmtk_element_pointer_enter(wlmtk_element_t *element_ptr)
-{
-    // TODO(kaeser@gubbe.ch): Consider forking this out into a 'leaf element'
-    // class. 'enter' and 'leave' don't make that much sense for eg. a
-    // container.
-    wlmtk_env_set_cursor(element_ptr->env_ptr, WLMTK_CURSOR_DEFAULT);
-}
-
-/* ------------------------------------------------------------------------- */
-/** Handler for when the pointer leaves the area. Nothing for default impl. */
-void _wlmtk_element_pointer_leave(__UNUSED__ wlmtk_element_t *element_ptr)
-{
-    // Nothing.
-}
-
 /* ------------------------------------------------------------------------- */
 /** Handler for losing keyboard focus. Nothing for default impl. */
 void _wlmtk_element_keyboard_blur(__UNUSED__ wlmtk_element_t *element_ptr)
@@ -377,14 +387,29 @@ void _wlmtk_element_keyboard_blur(__UNUSED__ wlmtk_element_t *element_ptr)
 /** Handler for keyboard events. By default: Nothing is handled. */
 bool _wlmtk_element_keyboard_event(
     __UNUSED__ wlmtk_element_t *element_ptr,
-    __UNUSED__ struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr,
-    __UNUSED__ const xkb_keysym_t *key_syms,
-    __UNUSED__ size_t key_syms_count,
+    __UNUSED__ struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr)
+{
+    return false;
+}
+
+/* ------------------------------------------------------------------------- */
+/** Handler for translated keys. By default: Nothing is handled. */
+bool _wlmtk_element_keyboard_sym(
+    __UNUSED__ wlmtk_element_t *element_ptr,
+    __UNUSED__ xkb_keysym_t keysym,
+    __UNUSED__ enum xkb_key_direction direction,
     __UNUSED__ uint32_t modifiers)
 {
     return false;
 }
 
+/* ------------------------------------------------------------------------- */
+/** Redraws the element contents. By default: Nothing is done. */
+void _wlmtk_element_layout(__UNUSED__ wlmtk_element_t *element_ptr)
+{
+    // Nothing.
+}
+
 /* ------------------------------------------------------------------------- */
 /**
  * Handles the 'destroy' callback of the wlr_scene_node.
@@ -415,51 +440,43 @@ static void fake_get_dimensions(
     int *top_ptr,
     int *right_ptr,
     int *bottom_ptr);
-static void fake_get_pointer_area(
+static bool fake_pointer_accepts_motion(
     wlmtk_element_t *element_ptr,
-    int *left_ptr,
-    int *top_ptr,
-    int *right_ptr,
-    int *bottom_ptr);
-static bool fake_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x, double y,
-    uint32_t time_msec);
+    wlmtk_pointer_motion_event_t *motion_event_ptr);
 static bool fake_pointer_button(
     wlmtk_element_t *element_ptr,
     const wlmtk_button_event_t *button_event_ptr);
 static bool fake_pointer_axis(
     wlmtk_element_t *element_ptr,
     struct wlr_pointer_axis_event *wlr_pointer_axis_event_ptr);
-static void fake_pointer_enter(
-    wlmtk_element_t *element_ptr);
-static void fake_pointer_leave(
-    wlmtk_element_t *element_ptr);
 static void fake_pointer_grab_cancel(
     wlmtk_element_t *element_ptr);
 static void fake_keyboard_blur(
     wlmtk_element_t *element_ptr);
 static bool fake_keyboard_event(
     wlmtk_element_t *element_ptr,
-    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr,
-    const xkb_keysym_t *key_syms,
-    size_t key_syms_count,
+    struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr);
+static bool fake_keyboard_sym(
+    wlmtk_element_t *element_ptr,
+    xkb_keysym_t keysym,
+    enum xkb_key_direction direction,
     uint32_t modifiers);
+static void fake_layout(
+    wlmtk_element_t *element_ptr);
 
 /** Virtual method table for the fake element. */
 static const wlmtk_element_vmt_t fake_element_vmt = {
     .destroy = fake_destroy,
     .create_scene_node = fake_create_scene_node,
     .get_dimensions = fake_get_dimensions,
-    .get_pointer_area = fake_get_pointer_area,
-    .pointer_motion = fake_pointer_motion,
+    .pointer_accepts_motion = fake_pointer_accepts_motion,
     .pointer_button = fake_pointer_button,
     .pointer_axis = fake_pointer_axis,
-    .pointer_enter = fake_pointer_enter,
-    .pointer_leave = fake_pointer_leave,
     .pointer_grab_cancel = fake_pointer_grab_cancel,
     .keyboard_blur = fake_keyboard_blur,
     .keyboard_event = fake_keyboard_event,
+    .keyboard_sym = fake_keyboard_sym,
+    .layout = fake_layout,
 };
 
 /* ------------------------------------------------------------------------- */
@@ -469,7 +486,7 @@ wlmtk_fake_element_t *wlmtk_fake_element_create(void)
         1, sizeof(wlmtk_fake_element_t));
     if (NULL == fake_element_ptr) return NULL;
 
-    if (!wlmtk_element_init(&fake_element_ptr->element, NULL)) {
+    if (!wlmtk_element_init(&fake_element_ptr->element)) {
         fake_destroy(&fake_element_ptr->element);
         return NULL;
     }
@@ -480,6 +497,19 @@ wlmtk_fake_element_t *wlmtk_fake_element_create(void)
     return fake_element_ptr;
 }
 
+/* ------------------------------------------------------------------------- */
+void wlmtk_fake_element_set_dimensions(
+    wlmtk_fake_element_t *fake_element_ptr,
+    int width, int height)
+{
+    fake_element_ptr->dimensions.width = width;
+    fake_element_ptr->dimensions.height = height;
+    if (NULL != fake_element_ptr->element.parent_container_ptr) {
+        wlmtk_container_invalidate_layout(
+            fake_element_ptr->element.parent_container_ptr);
+    }
+}
+
 /* ------------------------------------------------------------------------- */
 void wlmtk_fake_element_grab_keyboard(wlmtk_fake_element_t *fake_element_ptr)
 {
@@ -487,7 +517,8 @@ void wlmtk_fake_element_grab_keyboard(wlmtk_fake_element_t *fake_element_ptr)
     if (NULL != fake_element_ptr->element.parent_container_ptr) {
         wlmtk_container_set_keyboard_focus_element(
             fake_element_ptr->element.parent_container_ptr,
-            &fake_element_ptr->element);
+            &fake_element_ptr->element,
+            true);
     }
 }
 
@@ -526,44 +557,25 @@ void fake_get_dimensions(
     if (NULL != left_ptr) *left_ptr = fake_element_ptr->dimensions.x;
     if (NULL != top_ptr) *top_ptr = fake_element_ptr->dimensions.y;
     if (NULL != right_ptr) *right_ptr = (
-        fake_element_ptr->dimensions.width - fake_element_ptr->dimensions.x);
+        fake_element_ptr->dimensions.width + fake_element_ptr->dimensions.x);
     if (NULL != bottom_ptr) *bottom_ptr = (
-        fake_element_ptr->dimensions.height - fake_element_ptr->dimensions.y);
+        fake_element_ptr->dimensions.height + fake_element_ptr->dimensions.y);
 }
 
 /* ------------------------------------------------------------------------- */
-/** A "fake" 'get_pointer_area'. */
-void fake_get_pointer_area(
+/** Implements @ref wlmtk_element_vmt_t::pointer_accepts_motion. */
+bool fake_pointer_accepts_motion(
     wlmtk_element_t *element_ptr,
-    int *left_ptr,
-    int *top_ptr,
-    int *right_ptr,
-    int *bottom_ptr)
+    wlmtk_pointer_motion_event_t *motion_event_ptr)
 {
     wlmtk_fake_element_t *fake_element_ptr = BS_CONTAINER_OF(
         element_ptr, wlmtk_fake_element_t, element);
-    if (NULL != left_ptr) *left_ptr = -1;
-    if (NULL != top_ptr) *top_ptr = -2;
-    if (NULL != right_ptr) *right_ptr = (
-        fake_element_ptr->dimensions.width + 3);
-    if (NULL != bottom_ptr) *bottom_ptr = (
-        fake_element_ptr->dimensions.height + 4);
-}
 
-/* ------------------------------------------------------------------------- */
-/** Handles 'motion' events for the fake element, updates last position. */
-bool fake_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x,
-    double y,
-    uint32_t time_msec)
-{
-    wlmtk_fake_element_t *fake_element_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_fake_element_t, element);
-    fake_element_ptr->orig_vmt.pointer_motion(element_ptr, x, y, time_msec);
-    fake_element_ptr->pointer_motion_called = true;
-    return (-1 <= x && x < fake_element_ptr->dimensions.width + 3 &&
-            -2 < y && y < fake_element_ptr->dimensions.height + 4);
+    fake_element_ptr->pointer_accepts_motion_called = true;
+    return (-1 <= motion_event_ptr->x &&
+            motion_event_ptr->x < fake_element_ptr->dimensions.width + 3 &&
+            -2 < motion_event_ptr->y &&
+            motion_event_ptr->y < fake_element_ptr->dimensions.height + 4);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -576,10 +588,7 @@ bool fake_pointer_button(
         element_ptr, wlmtk_fake_element_t, element);
 
     fake_element_ptr->pointer_button_called = true;
-    memcpy(
-        &fake_element_ptr->pointer_button_event,
-        button_event_ptr,
-        sizeof(wlmtk_button_event_t));
+    fake_element_ptr->pointer_button_event = *button_event_ptr;
     return true;
 }
 
@@ -593,35 +602,10 @@ bool fake_pointer_axis(
         element_ptr, wlmtk_fake_element_t, element);
 
     fake_element_ptr->pointer_axis_called = true;
-    memcpy(
-        &fake_element_ptr->wlr_pointer_axis_event,
-        wlr_pointer_axis_event_ptr,
-        sizeof(struct wlr_pointer_axis_event));
+    fake_element_ptr->wlr_pointer_axis_event = *wlr_pointer_axis_event_ptr;
     return true;
 }
 
-/* ------------------------------------------------------------------------- */
-/** Handles 'enter' events for the fake element. */
-void fake_pointer_enter(
-    wlmtk_element_t *element_ptr)
-{
-    wlmtk_fake_element_t *fake_element_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_fake_element_t, element);
-    fake_element_ptr->orig_vmt.pointer_enter(element_ptr);
-    fake_element_ptr->pointer_enter_called = true;
-}
-
-/* ------------------------------------------------------------------------- */
-/** Handles 'leave' events for the fake element. */
-void fake_pointer_leave(
-    wlmtk_element_t *element_ptr)
-{
-    wlmtk_fake_element_t *fake_element_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_fake_element_t, element);
-    fake_element_ptr->orig_vmt.pointer_leave(element_ptr);
-    fake_element_ptr->pointer_leave_called = true;
-}
-
 /* ------------------------------------------------------------------------- */
 /** Records calls to @ref wlmtk_element_vmt_t::pointer_grab_cancel. */
 void fake_pointer_grab_cancel(
@@ -645,10 +629,7 @@ void fake_keyboard_blur(wlmtk_element_t *element_ptr)
 /** Handles 'keyboard_event' events for the fake element. */
 bool fake_keyboard_event(
     wlmtk_element_t *element_ptr,
-    __UNUSED__ struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr,
-    __UNUSED__ const xkb_keysym_t *key_syms,
-    __UNUSED__ size_t key_syms_count,
-    __UNUSED__ uint32_t modifiers)
+    __UNUSED__ struct wlr_keyboard_key_event *wlr_keyboard_key_event_ptr)
 {
     wlmtk_fake_element_t *fake_element_ptr = BS_CONTAINER_OF(
         element_ptr, wlmtk_fake_element_t, element);
@@ -656,30 +637,53 @@ bool fake_keyboard_event(
     return true;
 }
 
+/* ------------------------------------------------------------------------- */
+/** Handles 'keyboard_sym' events for the fake element. */
+bool fake_keyboard_sym(
+    wlmtk_element_t *element_ptr,
+    __UNUSED__ xkb_keysym_t keysym,
+    __UNUSED__ enum xkb_key_direction direction,
+    __UNUSED__ uint32_t modifiers)
+{
+    wlmtk_fake_element_t *fake_element_ptr = BS_CONTAINER_OF(
+        element_ptr, wlmtk_fake_element_t, element);
+    fake_element_ptr->keyboard_sym_called = true;
+    return true;
+}
+
+/* ------------------------------------------------------------------------- */
+/** Handles 'laoyut' calls for the fake element. */
+void fake_layout(wlmtk_element_t *element_ptr)
+{
+    wlmtk_fake_element_t *fake_element_ptr = BS_CONTAINER_OF(
+        element_ptr, wlmtk_fake_element_t, element);
+    fake_element_ptr->layout_called = true;
+}
+
 /* == Unit tests =========================================================== */
 
 static void test_init_fini(bs_test_t *test_ptr);
 static void test_set_parent_container(bs_test_t *test_ptr);
 static void test_set_get_position(bs_test_t *test_ptr);
 static void test_get_dimensions(bs_test_t *test_ptr);
-static void test_get_pointer_area(bs_test_t *test_ptr);
-static void test_pointer_motion_leave(bs_test_t *test_ptr);
+static void test_pointer_motion(bs_test_t *test_ptr);
 static void test_pointer_button(bs_test_t *test_ptr);
 static void test_pointer_axis(bs_test_t *test_ptr);
 static void test_keyboard_focus(bs_test_t *test_ptr);
-static void test_keyboard_event(bs_test_t *test_ptr);
+static void test_keyboard_activity(bs_test_t *test_ptr);
+static void test_layout(bs_test_t *test_ptr);
 
 const bs_test_case_t wlmtk_element_test_cases[] = {
     { 1, "init_fini", test_init_fini },
     { 1, "set_parent_container", test_set_parent_container },
     { 1, "set_get_position", test_set_get_position },
     { 1, "get_dimensions", test_get_dimensions },
-    { 1, "get_pointer_area", test_get_pointer_area },
-    { 1, "pointer_motion_leave", test_pointer_motion_leave },
+    { 1, "pointer_motion", test_pointer_motion },
     { 1, "pointer_button", test_pointer_button },
     { 1, "pointer_axis", test_pointer_axis },
     { 1, "keyboard_focus", test_keyboard_focus },
-    { 1, "keyboard_event", test_keyboard_event },
+    { 1, "keyboard_activity", test_keyboard_activity },
+    { 1, "layout", test_layout },
     { 0, NULL, NULL }
 };
 
@@ -688,7 +692,7 @@ const bs_test_case_t wlmtk_element_test_cases[] = {
 void test_init_fini(bs_test_t *test_ptr)
 {
     wlmtk_element_t element;
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_init(&element, NULL));
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_init(&element));
     wlmtk_element_extend(&element, &fake_element_vmt);
     BS_TEST_VERIFY_NEQ(test_ptr, NULL, element.vmt.destroy);
 
@@ -701,12 +705,12 @@ void test_init_fini(bs_test_t *test_ptr)
 void test_set_parent_container(bs_test_t *test_ptr)
 {
     wlmtk_element_t element;
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_init(&element, NULL));
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_init(&element));
     wlmtk_element_extend(&element, &fake_element_vmt);
 
     // Setting a parent without a scene graph tree will not set a node.
     wlmtk_container_t parent_no_tree;
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_container_init(&parent_no_tree, NULL));
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_container_init(&parent_no_tree));
     wlmtk_element_set_parent_container(&element, &parent_no_tree);
     BS_TEST_VERIFY_EQ(test_ptr, NULL, element.wlr_scene_node_ptr);
 
@@ -749,7 +753,7 @@ void test_set_parent_container(bs_test_t *test_ptr)
 void test_set_get_position(bs_test_t *test_ptr)
 {
     wlmtk_element_t element;
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_init(&element, NULL));
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_init(&element));
     wlmtk_element_extend(&element, &fake_element_vmt);
 
     // Exercise, must not crash.
@@ -805,8 +809,8 @@ void test_get_dimensions(bs_test_t *test_ptr)
         &fake_element_ptr->element, &top, &left, &right, &bottom);
     BS_TEST_VERIFY_EQ(test_ptr, -10, top);
     BS_TEST_VERIFY_EQ(test_ptr, -20, left);
-    BS_TEST_VERIFY_EQ(test_ptr, 52, right);
-    BS_TEST_VERIFY_EQ(test_ptr, 41, bottom);
+    BS_TEST_VERIFY_EQ(test_ptr, 32, right);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, bottom);
 
     struct wlr_box box = wlmtk_element_get_dimensions_box(
         &fake_element_ptr->element);
@@ -819,64 +823,82 @@ void test_get_dimensions(bs_test_t *test_ptr)
 }
 
 /* ------------------------------------------------------------------------- */
-/** Tests get_dimensions. */
-void test_get_pointer_area(bs_test_t *test_ptr)
-{
-    wlmtk_fake_element_t *fake_element_ptr = wlmtk_fake_element_create();
-    fake_element_ptr->dimensions.width = 42;
-    fake_element_ptr->dimensions.height = 21;
-
-    // Must not crash.
-    wlmtk_element_get_pointer_area(
-        &fake_element_ptr->element, NULL, NULL, NULL, NULL);
-
-    int top, left, right, bottom;
-    wlmtk_element_get_pointer_area(
-        &fake_element_ptr->element, &top, &left, &right, &bottom);
-    BS_TEST_VERIFY_EQ(test_ptr, -1, top);
-    BS_TEST_VERIFY_EQ(test_ptr, -2, left);
-    BS_TEST_VERIFY_EQ(test_ptr, 45, right);
-    BS_TEST_VERIFY_EQ(test_ptr, 25, bottom);
-
-    wlmtk_element_destroy(&fake_element_ptr->element);
-}
-
-/* ------------------------------------------------------------------------- */
-/** Exercises "pointer_motion" and "pointer_leave" methods. */
-void test_pointer_motion_leave(bs_test_t *test_ptr)
+/** Verifies that @ref wlmtk_element_pointer_motion raises signals. */
+void test_pointer_motion(bs_test_t *test_ptr)
 {
     wlmtk_fake_element_t *fake_element_ptr = wlmtk_fake_element_create();
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fake_element_ptr);
 
-    BS_TEST_VERIFY_TRUE(
-        test_ptr,
-        isnan(fake_element_ptr->element.last_pointer_x));
-    BS_TEST_VERIFY_TRUE(
-        test_ptr,
-        isnan(fake_element_ptr->element.last_pointer_y));
+    wlmtk_util_test_listener_t enter = {}, leave = {}, motion = {};
+    wlmtk_util_connect_test_listener(
+        &fake_element_ptr->element.events.pointer_enter, &enter);
+    wlmtk_util_connect_test_listener(
+        &fake_element_ptr->element.events.pointer_leave, &leave);
+    wlmtk_util_connect_test_listener(
+        &fake_element_ptr->element.events.pointer_motion, &motion);
 
-    wlmtk_element_pointer_motion(&fake_element_ptr->element, 1.0, 2.0, 1234);
-    BS_TEST_VERIFY_TRUE(test_ptr, fake_element_ptr->pointer_enter_called);
-    BS_TEST_VERIFY_EQ(test_ptr, 1.0, fake_element_ptr->element.last_pointer_x);
-    BS_TEST_VERIFY_EQ(test_ptr, 2.0, fake_element_ptr->element.last_pointer_y);
+    BS_TEST_VERIFY_FALSE(test_ptr, fake_element_ptr->element.pointer_inside);
+
+    // First motion, in the element. But invisible. Must not do anything.
+    wlmtk_pointer_motion_event_t e = { .x = 1.0, .y = 2.0, .time_msec = 3 };
+    BS_TEST_VERIFY_FALSE(
+        test_ptr,
+        wlmtk_element_pointer_motion(&fake_element_ptr->element, &e));
+    BS_TEST_VERIFY_EQ(test_ptr, 0, enter.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, 0, leave.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, 0, motion.calls);
+
+    // Now make it visible. Must trigger 'enter' and 'motion'.
+    wlmtk_element_set_visible(&fake_element_ptr->element, true);
+    e.pointer_ptr = (wlmtk_pointer_t*)0x1234;
+    BS_TEST_VERIFY_TRUE(
+        test_ptr,
+        wlmtk_element_pointer_motion(&fake_element_ptr->element, &e))
+    BS_TEST_VERIFY_EQ(test_ptr, 1, enter.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, e.pointer_ptr, enter.last_data_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 0, leave.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, motion.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, &e, motion.last_data_ptr);
+    BS_TEST_VERIFY_TRUE(test_ptr, fake_element_ptr->element.pointer_inside);
     BS_TEST_VERIFY_EQ(
         test_ptr,
-        1234,
-        fake_element_ptr->element.last_pointer_time_msec);
-
-    wlmtk_element_pointer_motion(&fake_element_ptr->element, NAN, NAN, 1235);
-    BS_TEST_VERIFY_TRUE(test_ptr, fake_element_ptr->pointer_leave_called);
-    BS_TEST_VERIFY_TRUE(
-        test_ptr,
-        isnan(fake_element_ptr->element.last_pointer_x));
-    BS_TEST_VERIFY_TRUE(
-        test_ptr,
-        isnan(fake_element_ptr->element.last_pointer_y));
+        1.0,
+        fake_element_ptr->element.last_pointer_motion_event.x);
     BS_TEST_VERIFY_EQ(
         test_ptr,
-        1235,
-        fake_element_ptr->element.last_pointer_time_msec);
+        2.0,
+        fake_element_ptr->element.last_pointer_motion_event.y);
+    BS_TEST_VERIFY_EQ(
+        test_ptr,
+        3,
+        fake_element_ptr->element.last_pointer_motion_event.time_msec);
 
+    // Another motion, within. Trigger just 'motion'.
+    e = (wlmtk_pointer_motion_event_t){ .x = 1.0, .y = 2.0, .time_msec = 4 };
+    BS_TEST_VERIFY_TRUE(
+        test_ptr,
+        wlmtk_element_pointer_motion(&fake_element_ptr->element, &e));
+    BS_TEST_VERIFY_EQ(test_ptr, 1, enter.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, 0, leave.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, 2, motion.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, &e, motion.last_data_ptr);
+    BS_TEST_VERIFY_TRUE(test_ptr, fake_element_ptr->element.pointer_inside);
+
+    // Another motion, outisde. Trigger 'leave'.
+    e = (wlmtk_pointer_motion_event_t){ .x = 10, .y = 20, .time_msec = 5 };
+    BS_TEST_VERIFY_FALSE(
+        test_ptr,
+        wlmtk_element_pointer_motion(&fake_element_ptr->element, &e));
+    BS_TEST_VERIFY_EQ(test_ptr, 1, enter.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, 1, leave.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, NULL, leave.last_data_ptr);
+    BS_TEST_VERIFY_EQ(test_ptr, 2, motion.calls);
+    BS_TEST_VERIFY_EQ(test_ptr, &e, motion.last_data_ptr);
+    BS_TEST_VERIFY_FALSE(test_ptr, fake_element_ptr->element.pointer_inside);
+
+    wlmtk_util_disconnect_test_listener(&motion);
+    wlmtk_util_disconnect_test_listener(&leave);
+    wlmtk_util_disconnect_test_listener(&enter);
     wlmtk_element_destroy(&fake_element_ptr->element);
 }
 
@@ -928,8 +950,8 @@ void test_keyboard_focus(bs_test_t *test_ptr)
 }
 
 /* ------------------------------------------------------------------------- */
-/** Exercises "keyboard_event" method. */
-void test_keyboard_event(bs_test_t *test_ptr)
+/** Exercises "keyboard_event" and "keyboard_sym" methods. */
+void test_keyboard_activity(bs_test_t *test_ptr)
 {
     wlmtk_fake_element_t *fake_element_ptr = wlmtk_fake_element_create();
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fake_element_ptr);
@@ -937,11 +959,31 @@ void test_keyboard_event(bs_test_t *test_ptr)
     struct wlr_keyboard_key_event event = {};
     BS_TEST_VERIFY_TRUE(
         test_ptr,
-        wlmtk_element_keyboard_event(
-            &fake_element_ptr->element, &event, NULL, 0, 0));
+        wlmtk_element_keyboard_event(&fake_element_ptr->element, &event));
     BS_TEST_VERIFY_TRUE(test_ptr, fake_element_ptr->keyboard_event_called);
+    BS_TEST_VERIFY_FALSE(test_ptr, fake_element_ptr->keyboard_sym_called);
+
+    fake_element_ptr->keyboard_event_called = false;
+    BS_TEST_VERIFY_TRUE(
+        test_ptr,
+        wlmtk_element_keyboard_sym(&fake_element_ptr->element, 0, 0, 0));
+    BS_TEST_VERIFY_FALSE(test_ptr, fake_element_ptr->keyboard_event_called);
+    BS_TEST_VERIFY_TRUE(test_ptr, fake_element_ptr->keyboard_sym_called);
 
     wlmtk_element_destroy(&fake_element_ptr->element);
 }
 
+/* ------------------------------------------------------------------------- */
+/** Exercises "layout" methods. */
+void test_layout(bs_test_t *test_ptr)
+{
+    wlmtk_fake_element_t *fake_element_ptr = wlmtk_fake_element_create();
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, fake_element_ptr);
+
+    wlmtk_element_layout(&fake_element_ptr->element);
+    BS_TEST_VERIFY_TRUE(test_ptr, fake_element_ptr->layout_called);
+    wlmtk_element_destroy(&fake_element_ptr->element);
+
+}
+
 /* == End of toolkit.c ===================================================== */
