@@ -20,7 +20,16 @@
 
 #include "menu_item.h"
 
-#include "gfxbuf.h"
+#include <cairo.h>
+#include <inttypes.h>
+#include <libbase/libbase.h>
+#include <linux/input-event-codes.h>
+#include <stdlib.h>
+
+#include "base.h"
+#include "buffer.h"
+#include "gfxbuf.h"  // IWYU pragma: keep
+#include "input.h"
 #include "primitives.h"
 #include "util.h"
 
@@ -44,6 +53,13 @@ struct _wlmtk_menu_item_t {
     /** Listens to @ref wlmtk_menu_events_t::open_changed. */
     struct wl_listener        submenu_open_changed_listener;
 
+    /** Listens to when we obtain pointer focus. */
+    struct wl_listener        pointer_enter_listener;
+    /** Listens to when we lose pointer focus. */
+    struct wl_listener        pointer_leave_listener;
+    /** Listens to when we get pointer motion. To re-gain mouse mode. */
+    struct wl_listener        pointer_motion_listener;
+
     /** List node, within @ref wlmtk_menu_t::items. */
     bs_dllist_node_t          dlnode;
 
@@ -52,7 +68,7 @@ struct _wlmtk_menu_item_t {
     /** Width of the item element, in pixels. */
     int                       width;
     /** Mode of the menu (and the item). */
-    wlmtk_menu_mode_t         mode;
+    enum wlmtk_menu_mode      mode;
 
     /** Texture buffer holding the item in enabled state. */
     struct wlr_buffer         *enabled_wlr_buffer_ptr;
@@ -80,21 +96,28 @@ static void _wlmtk_menu_item_draw_state(wlmtk_menu_item_t *menu_item_ptr);
 static struct wlr_buffer *_wlmtk_menu_item_create_buffer(
     wlmtk_menu_item_t *menu_item_ptr,
     wlmtk_menu_item_state_t state);
+static void _wlmtk_menu_item_draw_submenu_hint(
+    cairo_t *cairo_ptr,
+    const wlmtk_menu_item_style_t *style_ptr,
+    double x,
+    double y);
 
-static bool _wlmtk_menu_item_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x, double y,
-    uint32_t time_msec);
 static bool _wlmtk_menu_item_element_pointer_button(
     wlmtk_element_t *element_ptr,
     const wlmtk_button_event_t *button_event_ptr);
-static void _wlmtk_menu_item_element_pointer_enter(
-    wlmtk_element_t *element_ptr);
-static void _wlmtk_menu_item_element_pointer_leave(
-    wlmtk_element_t *element_ptr);
 static void _wlmtk_menu_item_element_destroy(
     wlmtk_element_t *element_ptr);
 
+static void _wlmtk_menu_item_handle_pointer_enter(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+static void _wlmtk_menu_item_handle_pointer_leave(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+static void _wlmtk_menu_item_handle_pointer_motion(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
+
 static void _wlmtk_menu_item_handle_open_changed(
     struct wl_listener *listener_ptr,
     void *data_ptr);
@@ -103,10 +126,7 @@ static void _wlmtk_menu_item_handle_open_changed(
 
 /** Virtual method table for the menu item's super class: Element. */
 static const wlmtk_element_vmt_t _wlmtk_menu_item_element_vmt = {
-    .pointer_motion = _wlmtk_menu_item_element_pointer_motion,
     .pointer_button = _wlmtk_menu_item_element_pointer_button,
-    .pointer_enter = _wlmtk_menu_item_element_pointer_enter,
-    .pointer_leave = _wlmtk_menu_item_element_pointer_leave,
     .destroy = _wlmtk_menu_item_element_destroy,
 };
 
@@ -133,8 +153,7 @@ static const wlmtk_menu_item_style_t _item_test_style = {
 
 /* -------------------------------------------------------------------------*/
 wlmtk_menu_item_t *wlmtk_menu_item_create(
-    const wlmtk_menu_item_style_t *style_ptr,
-    wlmtk_env_t *env_ptr)
+    const wlmtk_menu_item_style_t *style_ptr)
 {
     wlmtk_menu_item_t *menu_item_ptr = logged_calloc(
         1, sizeof(wlmtk_menu_item_t));
@@ -142,7 +161,7 @@ wlmtk_menu_item_t *wlmtk_menu_item_create(
     wl_signal_init(&menu_item_ptr->events.triggered);
     wl_signal_init(&menu_item_ptr->events.destroy);
 
-    if (!wlmtk_buffer_init(&menu_item_ptr->super_buffer, env_ptr)) {
+    if (!wlmtk_buffer_init(&menu_item_ptr->super_buffer)) {
         wlmtk_menu_item_destroy(menu_item_ptr);
         return NULL;
     }
@@ -150,6 +169,19 @@ wlmtk_menu_item_t *wlmtk_menu_item_create(
         &menu_item_ptr->super_buffer.super_element,
         &_wlmtk_menu_item_element_vmt);
 
+    wlmtk_util_connect_listener_signal(
+        &menu_item_ptr->super_buffer.super_element.events.pointer_enter,
+        &menu_item_ptr->pointer_enter_listener,
+        _wlmtk_menu_item_handle_pointer_enter);
+    wlmtk_util_connect_listener_signal(
+        &menu_item_ptr->super_buffer.super_element.events.pointer_leave,
+        &menu_item_ptr->pointer_leave_listener,
+        _wlmtk_menu_item_handle_pointer_leave);
+    wlmtk_util_connect_listener_signal(
+        &menu_item_ptr->super_buffer.super_element.events.pointer_motion,
+        &menu_item_ptr->pointer_motion_listener,
+        _wlmtk_menu_item_handle_pointer_motion);
+
     menu_item_ptr->style = *style_ptr;
     // TODO(kaeser@gubbe.ch): Should not be required!
     menu_item_ptr->width = style_ptr->width;
@@ -167,13 +199,17 @@ void wlmtk_menu_item_destroy(wlmtk_menu_item_t *menu_item_ptr)
 {
     wl_signal_emit(&menu_item_ptr->events.destroy, NULL);
 
+    wlmtk_util_disconnect_listener(&menu_item_ptr->pointer_motion_listener);
+    wlmtk_util_disconnect_listener(&menu_item_ptr->pointer_leave_listener);
+    wlmtk_util_disconnect_listener(&menu_item_ptr->pointer_enter_listener);
+
     if (NULL != menu_item_ptr->submenu_ptr) {
         wlmtk_util_disconnect_listener(
             &menu_item_ptr->submenu_open_changed_listener);
         if (NULL != menu_item_ptr->menu_ptr) {
-            wlmtk_pane_remove_popup(
-                wlmtk_menu_pane(menu_item_ptr->menu_ptr),
-                wlmtk_menu_pane(menu_item_ptr->submenu_ptr));
+            wlmtk_base_push_element(
+                wlmtk_menu_base(menu_item_ptr->menu_ptr),
+                wlmtk_menu_element(menu_item_ptr->submenu_ptr));
         }
 
         wlmtk_menu_destroy(menu_item_ptr->submenu_ptr);
@@ -209,17 +245,17 @@ void wlmtk_menu_item_set_parent_menu(
 
     if (NULL != menu_item_ptr->menu_ptr &&
         NULL != menu_item_ptr->submenu_ptr) {
-        wlmtk_pane_remove_popup(
-            wlmtk_menu_pane(menu_item_ptr->menu_ptr),
-            wlmtk_menu_pane(menu_item_ptr->submenu_ptr));
+        wlmtk_base_pop_element(
+            wlmtk_menu_base(menu_item_ptr->menu_ptr),
+            wlmtk_menu_element(menu_item_ptr->submenu_ptr));
     }
 
     menu_item_ptr->menu_ptr = menu_ptr;
     if (NULL != menu_item_ptr->menu_ptr &&
         NULL != menu_item_ptr->submenu_ptr) {
-        wlmtk_pane_add_popup(
-            wlmtk_menu_pane(menu_item_ptr->menu_ptr),
-            wlmtk_menu_pane(menu_item_ptr->submenu_ptr));
+        wlmtk_base_push_element(
+            wlmtk_menu_base(menu_item_ptr->menu_ptr),
+            wlmtk_menu_element(menu_item_ptr->submenu_ptr));
     }
 }
 
@@ -235,10 +271,11 @@ void wlmtk_menu_item_set_submenu(
             &menu_item_ptr->submenu_open_changed_listener);
 
         if (NULL != menu_item_ptr->menu_ptr) {
-            wlmtk_pane_remove_popup(
-                wlmtk_menu_pane(menu_item_ptr->menu_ptr),
-                wlmtk_menu_pane(menu_item_ptr->submenu_ptr));
+            wlmtk_base_pop_element(
+                wlmtk_menu_base(menu_item_ptr->menu_ptr),
+                wlmtk_menu_element(menu_item_ptr->submenu_ptr));
         }
+        wlmtk_menu_set_parent_item(submenu_ptr, NULL);
     }
 
     menu_item_ptr->submenu_ptr = submenu_ptr;
@@ -249,19 +286,29 @@ void wlmtk_menu_item_set_submenu(
             _wlmtk_menu_item_handle_open_changed);
 
         if (NULL != menu_item_ptr->menu_ptr) {
-            wlmtk_pane_add_popup(
-                wlmtk_menu_pane(menu_item_ptr->menu_ptr),
-                wlmtk_menu_pane(menu_item_ptr->submenu_ptr));
+            wlmtk_base_push_element(
+                wlmtk_menu_base(menu_item_ptr->menu_ptr),
+                wlmtk_menu_element(menu_item_ptr->submenu_ptr));
         }
 
         wlmtk_menu_set_mode(menu_item_ptr->submenu_ptr, menu_item_ptr->mode);
+        wlmtk_menu_set_parent_item(submenu_ptr, menu_item_ptr);
     }
+
+    _wlmtk_menu_item_redraw(menu_item_ptr);
+}
+
+/* ------------------------------------------------------------------------- */
+wlmtk_menu_t *wlmtk_menu_item_get_submenu(
+    wlmtk_menu_item_t *menu_item_ptr)
+{
+    return menu_item_ptr->submenu_ptr;
 }
 
 /* ------------------------------------------------------------------------- */
 void wlmtk_menu_item_set_mode(
     wlmtk_menu_item_t *menu_item_ptr,
-    wlmtk_menu_mode_t mode)
+    enum wlmtk_menu_mode mode)
 {
     menu_item_ptr->mode = mode;
     if (NULL != menu_item_ptr->submenu_ptr) {
@@ -270,7 +317,7 @@ void wlmtk_menu_item_set_mode(
 }
 
 /* ------------------------------------------------------------------------- */
-wlmtk_menu_mode_t wlmtk_menu_item_get_mode(
+enum wlmtk_menu_mode wlmtk_menu_item_get_mode(
     wlmtk_menu_item_t *menu_item_ptr)
 {
     return menu_item_ptr->mode;
@@ -339,16 +386,39 @@ bool wlmtk_menu_item_set_highlighted(
     return true;
 }
 
+/* -------------------------------------------------------------------------*/
+void wlmtk_menu_item_trigger(wlmtk_menu_item_t *menu_item_ptr)
+{
+    switch (menu_item_ptr->state) {
+    case WLMTK_MENU_ITEM_ENABLED:
+    case WLMTK_MENU_ITEM_HIGHLIGHTED:
+
+        wl_signal_emit(&menu_item_ptr->events.triggered, NULL);
+        break;
+
+    case WLMTK_MENU_ITEM_DISABLED:
+    default:
+        bs_log(BS_WARNING, "Ignoring trigger for item %p (\"%s\"), state %d",
+               menu_item_ptr,
+               menu_item_ptr->text_ptr ? menu_item_ptr->text_ptr : "",
+               menu_item_ptr->state);
+    }
+}
+
 /* -------------------------------------------------------------------------*/
 bs_dllist_node_t *wlmtk_dlnode_from_menu_item(
     wlmtk_menu_item_t *menu_item_ptr)
 {
+    // Guard clause.
+    if (NULL == menu_item_ptr) return NULL;
     return &menu_item_ptr->dlnode;
 }
 
 /* -------------------------------------------------------------------------*/
 wlmtk_menu_item_t *wlmtk_menu_item_from_dlnode(bs_dllist_node_t *dlnode_ptr)
 {
+    // Guard clause.
+    if (NULL == dlnode_ptr) return NULL;
     return BS_CONTAINER_OF(dlnode_ptr, wlmtk_menu_item_t, dlnode);
 }
 
@@ -488,6 +558,15 @@ struct wlr_buffer *_wlmtk_menu_item_create_buffer(
     wlmaker_primitives_cairo_fill(cairo_ptr, fill_ptr);
     wlmaker_primitives_draw_bezel(
         cairo_ptr, menu_item_ptr->style.bezel_width, true);
+
+    if (NULL != menu_item_ptr->submenu_ptr) {
+        _wlmtk_menu_item_draw_submenu_hint(
+            cairo_ptr,
+            &menu_item_ptr->style,
+            menu_item_ptr->width - menu_item_ptr->style.height * 0.6,
+            menu_item_ptr->style.height * 0.3);
+    }
+
     wlmaker_primitives_draw_text(
         cairo_ptr,
         6, 2 + menu_item_ptr->style.font.size,
@@ -500,24 +579,41 @@ struct wlr_buffer *_wlmtk_menu_item_create_buffer(
 }
 
 /* ------------------------------------------------------------------------- */
-/** Requests this item to be highlighted if there's motion. */
-bool _wlmtk_menu_item_element_pointer_motion(
-    wlmtk_element_t *element_ptr,
-    double x, double y,
-    uint32_t time_msec)
+/**
+ * Draws the hint for submenu (triangle) into the cairo at (x, y).
+ *
+ * @param cairo_ptr
+ * @param style_ptr
+ * @param x
+ * @param y
+ */
+void _wlmtk_menu_item_draw_submenu_hint(
+    cairo_t *cairo_ptr,
+    const wlmtk_menu_item_style_t *style_ptr,
+    double x,
+    double y)
 {
-    wlmtk_menu_item_t *menu_item_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_menu_item_t, super_buffer.super_element);
-    bool rv = menu_item_ptr->orig_super_element_vmt.pointer_motion(
-        element_ptr, x, y, time_msec);
+    double h = style_ptr->height;
+    cairo_save(cairo_ptr);
+    cairo_set_line_cap(cairo_ptr, CAIRO_LINE_CAP_BUTT);
+    cairo_set_line_width(cairo_ptr, style_ptr->bezel_width);
+    cairo_move_to(cairo_ptr, x, y);
 
-    if (rv && menu_item_ptr->enabled && NULL != menu_item_ptr->menu_ptr) {
-        wlmtk_menu_request_item_highlight(
-            menu_item_ptr->menu_ptr,
-            menu_item_ptr);
-    }
+    cairo_set_source_rgba(cairo_ptr, 0.0, 0.0, 0.0, 0.4);
+    cairo_line_to(cairo_ptr, x, h * 0.7);
+    cairo_stroke(cairo_ptr);
 
-    return rv;
+    cairo_set_source_rgba(cairo_ptr, 1.0, 1.0, 1.0, 0.8);
+    cairo_move_to(cairo_ptr, x, h * 0.7);
+    cairo_line_to(cairo_ptr, x + h * 0.4, h * 0.5);
+    cairo_stroke(cairo_ptr);
+
+    cairo_set_source_rgba(cairo_ptr, 0.5, 0.5, 0.5, 0.7);
+    cairo_move_to(cairo_ptr, x + h * 0.4, h * 0.5);
+    cairo_line_to(cairo_ptr, x, y);
+    cairo_stroke(cairo_ptr);
+
+    cairo_restore(cairo_ptr);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -535,7 +631,7 @@ bool _wlmtk_menu_item_element_pointer_button(
         WLMTK_BUTTON_CLICK == button_event_ptr->type &&
         WLMTK_MENU_ITEM_DISABLED != menu_item_ptr->state &&
         wlmtk_menu_item_element(menu_item_ptr)->pointer_inside) {
-        wl_signal_emit(&menu_item_ptr->events.triggered, NULL);
+        wlmtk_menu_item_trigger(menu_item_ptr);
         return true;
     }
 
@@ -545,7 +641,7 @@ bool _wlmtk_menu_item_element_pointer_button(
         WLMTK_BUTTON_UP == button_event_ptr->type &&
         WLMTK_MENU_ITEM_DISABLED != menu_item_ptr->state &&
         wlmtk_menu_item_element(menu_item_ptr)->pointer_inside) {
-        wl_signal_emit(&menu_item_ptr->events.triggered, NULL);
+        wlmtk_menu_item_trigger(menu_item_ptr);
         return true;
     }
 
@@ -554,13 +650,24 @@ bool _wlmtk_menu_item_element_pointer_button(
 }
 
 /* ------------------------------------------------------------------------- */
-/** Handles when the pointer enters the element: Highlights, if enabled. */
-void _wlmtk_menu_item_element_pointer_enter(
+/** Implements @ref wlmtk_element_vmt_t::destroy. Dtor for the menu item. */
+void _wlmtk_menu_item_element_destroy(
     wlmtk_element_t *element_ptr)
 {
     wlmtk_menu_item_t *menu_item_ptr = BS_CONTAINER_OF(
         element_ptr, wlmtk_menu_item_t, super_buffer.super_element);
-    menu_item_ptr->orig_super_element_vmt.pointer_enter(element_ptr);
+
+    wlmtk_menu_item_destroy(menu_item_ptr);
+}
+
+/* ------------------------------------------------------------------------- */
+/** Handles when the pointer enters the element: Highlights, if enabled. */
+void _wlmtk_menu_item_handle_pointer_enter(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
+{
+    wlmtk_menu_item_t *menu_item_ptr = BS_CONTAINER_OF(
+        listener_ptr, wlmtk_menu_item_t, pointer_enter_listener);
 
     if (menu_item_ptr->enabled && NULL != menu_item_ptr->menu_ptr) {
         wlmtk_menu_request_item_highlight(
@@ -574,14 +681,15 @@ void _wlmtk_menu_item_element_pointer_enter(
  * Handles when the pointer leaves the element: Ends highlight, in case there
  * is no submenu currently visible.
  *
- * @param element_ptr
+ * @param listener_ptr
+ * @param data_ptr
  */
-void _wlmtk_menu_item_element_pointer_leave(
-    wlmtk_element_t *element_ptr)
+void _wlmtk_menu_item_handle_pointer_leave(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
 {
     wlmtk_menu_item_t *menu_item_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_menu_item_t, super_buffer.super_element);
-    menu_item_ptr->orig_super_element_vmt.pointer_leave(element_ptr);
+        listener_ptr, wlmtk_menu_item_t, pointer_leave_listener);
 
     if (menu_item_ptr->enabled &&
         WLMTK_MENU_ITEM_HIGHLIGHTED == menu_item_ptr->state &&
@@ -593,14 +701,24 @@ void _wlmtk_menu_item_element_pointer_leave(
 }
 
 /* ------------------------------------------------------------------------- */
-/** Implements @ref wlmtk_element_vmt_t::destroy. Dtor for the menu item. */
-void _wlmtk_menu_item_element_destroy(
-    wlmtk_element_t *element_ptr)
+/**
+ * Handles when there is pointer motion within the element. (Re)gain highlight.
+ *
+ * @param listener_ptr
+ * @param data_ptr
+ */
+void _wlmtk_menu_item_handle_pointer_motion(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
 {
     wlmtk_menu_item_t *menu_item_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_menu_item_t, super_buffer.super_element);
+        listener_ptr, wlmtk_menu_item_t, pointer_motion_listener);
 
-    wlmtk_menu_item_destroy(menu_item_ptr);
+    if (menu_item_ptr->enabled && NULL != menu_item_ptr->menu_ptr) {
+        wlmtk_menu_request_item_highlight(
+            menu_item_ptr->menu_ptr,
+            menu_item_ptr);
+    }
 }
 
 /* ------------------------------------------------------------------------- */
@@ -648,7 +766,7 @@ const bs_test_case_t wlmtk_menu_item_test_cases[] = {
 void test_create_destroy(bs_test_t *test_ptr)
 {
     wlmtk_menu_item_t *item_ptr = wlmtk_menu_item_create(
-        &_item_test_style, NULL);
+        &_item_test_style);
     BS_TEST_VERIFY_TRUE_OR_RETURN(test_ptr, item_ptr);
 
     bs_dllist_node_t *dlnode_ptr = wlmtk_dlnode_from_menu_item(item_ptr);
@@ -665,6 +783,9 @@ void test_create_destroy(bs_test_t *test_ptr)
 
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_menu_item_set_text(item_ptr, "Text"));
 
+    BS_TEST_VERIFY_EQ(test_ptr, NULL, wlmtk_menu_item_from_dlnode(NULL));
+    BS_TEST_VERIFY_EQ(test_ptr, NULL, wlmtk_dlnode_from_menu_item(NULL));
+
     wlmtk_menu_item_destroy(item_ptr);
 }
 
@@ -673,7 +794,7 @@ void test_create_destroy(bs_test_t *test_ptr)
 void test_buffers(bs_test_t *test_ptr)
 {
     wlmtk_menu_item_t *item_ptr = wlmtk_menu_item_create(
-        &_item_test_style, NULL);
+        &_item_test_style);
     BS_TEST_VERIFY_TRUE_OR_RETURN(test_ptr, item_ptr);
 
     item_ptr->width = 80;
@@ -693,6 +814,23 @@ void test_buffers(bs_test_t *test_ptr)
     BS_TEST_VERIFY_GFXBUF_EQUALS_PNG(
         test_ptr, g, "toolkit/menu_item_disabled.png");
 
+    wlmtk_menu_style_t s = {};
+    wlmtk_menu_t *submenu_ptr = wlmtk_menu_create(&s);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, submenu_ptr);
+    wlmtk_menu_item_set_submenu(item_ptr, submenu_ptr);
+
+    g = bs_gfxbuf_from_wlr_buffer(item_ptr->enabled_wlr_buffer_ptr);
+    BS_TEST_VERIFY_GFXBUF_EQUALS_PNG(
+        test_ptr, g, "toolkit/menu_item_submenu_enabled.png");
+
+    g = bs_gfxbuf_from_wlr_buffer(item_ptr->highlighted_wlr_buffer_ptr);
+    BS_TEST_VERIFY_GFXBUF_EQUALS_PNG(
+        test_ptr, g, "toolkit/menu_item_submenu_highlighted.png");
+
+    g = bs_gfxbuf_from_wlr_buffer(item_ptr->disabled_wlr_buffer_ptr);
+    BS_TEST_VERIFY_GFXBUF_EQUALS_PNG(
+        test_ptr, g, "toolkit/menu_item_submenu_disabled.png");
+
     wlmtk_menu_item_destroy(item_ptr);
 }
 
@@ -701,10 +839,9 @@ void test_buffers(bs_test_t *test_ptr)
 void test_pointer(bs_test_t *test_ptr)
 {
     wlmtk_menu_style_t s = {};
-    wlmtk_menu_t *menu_ptr = wlmtk_menu_create(&s, NULL);
+    wlmtk_menu_t *menu_ptr = wlmtk_menu_create(&s);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, menu_ptr);
-    wlmtk_menu_item_t *item_ptr = wlmtk_menu_item_create(
-        &_item_test_style, NULL);
+    wlmtk_menu_item_t *item_ptr = wlmtk_menu_item_create(&_item_test_style);
     BS_TEST_VERIFY_TRUE_OR_RETURN(test_ptr, item_ptr);
     wlmtk_menu_add_item(menu_ptr, item_ptr);
     BS_TEST_VERIFY_EQ(test_ptr, menu_ptr, item_ptr->menu_ptr);
@@ -742,7 +879,8 @@ void test_pointer(bs_test_t *test_ptr)
         item_ptr->disabled_wlr_buffer_ptr);
 
     // Pointer enters the item, but remains disabled.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, 20, 10, 1));
+    wlmtk_pointer_motion_event_t mev = { .x = 20, .y = 10 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, &mev));
     BS_TEST_VERIFY_EQ(
         test_ptr,
         WLMTK_MENU_ITEM_DISABLED,
@@ -766,7 +904,8 @@ void test_pointer(bs_test_t *test_ptr)
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_button(e, &lbtn_ev));
 
     // Pointer moves outside: disabled.
-    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_element_pointer_motion(e, 90, 10, 2));
+    mev = (wlmtk_pointer_motion_event_t){ .x = 90, .y = 10 };
+    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_element_pointer_motion(e, &mev));
     BS_TEST_VERIFY_EQ(
         test_ptr,
         WLMTK_MENU_ITEM_ENABLED,
@@ -786,8 +925,7 @@ void test_pointer(bs_test_t *test_ptr)
 /** Verifies desired clicks are passed to the handler. */
 void test_triggered(bs_test_t *test_ptr)
 {
-    wlmtk_menu_item_t *item_ptr = wlmtk_menu_item_create(
-        &_item_test_style, NULL);
+    wlmtk_menu_item_t *item_ptr = wlmtk_menu_item_create(&_item_test_style);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, item_ptr);
     wlmtk_util_test_listener_t tl;
     wlmtk_util_connect_test_listener(
@@ -798,18 +936,21 @@ void test_triggered(bs_test_t *test_ptr)
     wlmtk_button_event_t b = { .button = BTN_LEFT, .type = WLMTK_BUTTON_CLICK };
 
     // Pointer enters to highlight, the click triggers the handler.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, 20, 10, 1));
+    wlmtk_pointer_motion_event_t mev = { .x = 20, .y = 10 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, &mev));
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_button(e, &b));
     BS_TEST_VERIFY_EQ(test_ptr, 1, tl.calls);
     wlmtk_util_clear_test_listener(&tl);
 
     // Pointer enters outside, click does not trigger.
-    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_element_pointer_motion(e, 90, 10, 1));
+    mev = (wlmtk_pointer_motion_event_t){ .x = 90, .y = 10 };
+    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_element_pointer_motion(e, &mev));
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_button(e, &b));
     BS_TEST_VERIFY_EQ(test_ptr, 0, tl.calls);
 
     // Pointer enters again. Element disabled, will not trigger.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, 20, 10, 1));
+    mev = (wlmtk_pointer_motion_event_t){ .x = 20, .y = 10 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, &mev));
     wlmtk_menu_item_set_enabled(item_ptr, false);
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_button(e, &b));
     BS_TEST_VERIFY_EQ(test_ptr, 0, tl.calls);
@@ -845,8 +986,7 @@ void test_triggered(bs_test_t *test_ptr)
 /** Tests button events in right-click mode. */
 void test_right_click(bs_test_t *test_ptr)
 {
-    wlmtk_menu_item_t *item_ptr = wlmtk_menu_item_create(
-        &_item_test_style, NULL);
+    wlmtk_menu_item_t *item_ptr = wlmtk_menu_item_create(&_item_test_style);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, item_ptr);
     wlmtk_util_test_listener_t tl;
     wlmtk_util_connect_test_listener(
@@ -863,7 +1003,8 @@ void test_right_click(bs_test_t *test_ptr)
         wlmtk_menu_item_get_mode(item_ptr));
 
     // Pointer enters to highlight, click triggers.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, 20, 10, 1));
+    wlmtk_pointer_motion_event_t mev = { .x = 20, .y = 10 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, &mev));
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_button(e, &b));
     BS_TEST_VERIFY_EQ(test_ptr, 1, tl.calls);
     wlmtk_util_clear_test_listener(&tl);
@@ -880,7 +1021,8 @@ void test_right_click(bs_test_t *test_ptr)
         wlmtk_menu_item_get_mode(item_ptr));
 
     // Pointer remains inside, click is ignored.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, 20, 10, 1));
+    mev = (wlmtk_pointer_motion_event_t){ .x = 20, .y = 10 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(e, &mev));
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_button(e, &b));
     BS_TEST_VERIFY_EQ(test_ptr, 0, tl.calls);
 
@@ -890,7 +1032,8 @@ void test_right_click(bs_test_t *test_ptr)
     wlmtk_util_clear_test_listener(&tl);
 
     // Pointer leaves, button-up does not trigger.
-    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_element_pointer_motion(e, 90, 10, 1));
+    mev = (wlmtk_pointer_motion_event_t){ .x = 90, .y = 10 };
+    BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_element_pointer_motion(e, &mev));
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_button(e, &b));
     BS_TEST_VERIFY_EQ(test_ptr, 0, tl.calls);
 
@@ -913,52 +1056,60 @@ void test_submenu_highlight(bs_test_t *test_ptr)
 {
     wlmtk_menu_style_t s = { .item = _item_test_style };
 
-    wlmtk_menu_t *menu_ptr = wlmtk_menu_create(&s, NULL);
+    wlmtk_menu_t *menu_ptr = wlmtk_menu_create(&s);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, menu_ptr);
     wlmtk_menu_set_mode(menu_ptr, WLMTK_MENU_MODE_RIGHTCLICK);
     wlmtk_element_t *me = wlmtk_menu_element(menu_ptr);
+    wlmtk_element_set_visible(me, true);
 
-    wlmtk_menu_item_t *i1 = wlmtk_menu_item_create(&_item_test_style, NULL);
+    wlmtk_menu_item_t *i1 = wlmtk_menu_item_create(&_item_test_style);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, i1);
     wlmtk_menu_add_item(menu_ptr, i1);
-    wlmtk_menu_item_t *i2 = wlmtk_menu_item_create(&_item_test_style, NULL);
+    wlmtk_menu_item_t *i2 = wlmtk_menu_item_create(&_item_test_style);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, i2);
     wlmtk_menu_add_item(menu_ptr, i2);
 
-    wlmtk_menu_t *submenu_ptr = wlmtk_menu_create(&s, NULL);
+    wlmtk_menu_t *submenu_ptr = wlmtk_menu_create(&s);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, submenu_ptr);
-    wlmtk_menu_item_t *s1 = wlmtk_menu_item_create(&_item_test_style, NULL);
+    wlmtk_menu_item_t *s1 = wlmtk_menu_item_create(&_item_test_style);
     BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, s1);
     wlmtk_menu_add_item(submenu_ptr, s1);
+    wlmtk_element_layout(wlmtk_menu_element(menu_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, NULL, wlmtk_menu_get_parent_item(submenu_ptr));
     wlmtk_menu_item_set_submenu(i2, submenu_ptr);
     BS_TEST_VERIFY_EQ(
         test_ptr,
         WLMTK_MENU_MODE_RIGHTCLICK,
         wlmtk_menu_get_mode(submenu_ptr));
+    BS_TEST_VERIFY_EQ(test_ptr, i2, wlmtk_menu_get_parent_item(submenu_ptr));
 
     // Begin: Move pointer so that i1 is highlighted.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, 9, 12, 1));
+    wlmtk_pointer_motion_event_t mev = { .x = 9, .y = 12 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, &mev));
     BS_TEST_VERIFY_EQ(
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(i1));
     BS_TEST_VERIFY_NEQ(
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(i2));
 
     // Then: move pointer into i2. Must highlight and open submenu.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, 9, 36, 1));
+    mev = (wlmtk_pointer_motion_event_t){ .x = 9, .y = 36 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, &mev));
     BS_TEST_VERIFY_NEQ(
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(i1));
     BS_TEST_VERIFY_EQ(
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(i2));
 
     // Then: Move pointer into i2 and submenu. Must highlight the submenu item.
+    mev = (wlmtk_pointer_motion_event_t){ .x = 209, .y = 36 };
     BS_TEST_VERIFY_EQ(
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(i2));
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, 209, 36, 1));
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, &mev));
     BS_TEST_VERIFY_EQ(
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(s1));
 
     // Then: Move pointer a bit, within i1. Highlight that, close submenu.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, 9, 13, 1));
+    mev = (wlmtk_pointer_motion_event_t){ .x = 9, .y = 13 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, &mev));
     BS_TEST_VERIFY_FALSE(test_ptr, wlmtk_menu_is_open(submenu_ptr));
     BS_TEST_VERIFY_EQ(
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(i1));
@@ -966,7 +1117,8 @@ void test_submenu_highlight(bs_test_t *test_ptr)
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(s1));
 
     // Then: Move pointer into i2. Must highlight and (re)open submenu.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, 9, 36, 1));
+    mev = (wlmtk_pointer_motion_event_t){ .x = 9, .y = 36 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, &mev));
     BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_menu_is_open(submenu_ptr));
     BS_TEST_VERIFY_EQ(
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(i2));
@@ -974,7 +1126,8 @@ void test_submenu_highlight(bs_test_t *test_ptr)
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(s1));
 
     // Then: Move pointer into submenu again. Release button. Must trigger.
-    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, 209, 36, 1));
+    mev = (wlmtk_pointer_motion_event_t){ .x = 209, .y = 36 };
+    BS_TEST_VERIFY_TRUE(test_ptr, wlmtk_element_pointer_motion(me, &mev));
     BS_TEST_VERIFY_EQ(
         test_ptr, WLMTK_MENU_ITEM_HIGHLIGHTED, wlmtk_menu_item_get_state(s1));
 
