@@ -31,6 +31,7 @@
 
 #include "container.h"
 #include "input.h"
+#include "output_tracker.h"
 #include "rectangle.h"
 #include "test.h"  // IWYU pragma: keep
 #include "tile.h"
@@ -72,6 +73,8 @@ struct _wlmtk_root_t {
 
     /** Listener for wlr_output_layout::events.change. */
     struct wl_listener        output_layout_change_listener;
+    /** Output layout tracker. */
+    wlmtk_output_tracker_t    *output_tracker_ptr;
 
     // Elements below not owned by wlmtk_root_t.
     /** wlroots output layout. */
@@ -81,6 +84,14 @@ struct _wlmtk_root_t {
     wlmtk_pointer_motion_event_t mev;
 };
 
+/** Listeners for a specific output. */
+struct wlmtk_root_output {
+    /** Indicates this output is ready to render a frame. */
+    struct wl_listener        frame_listener;
+    /** Back-link to root. */
+    wlmtk_root_t              *root_ptr;
+};
+
 static void _wlmtk_root_switch_to_workspace(
     wlmtk_root_t *root_ptr,
     wlmtk_workspace_t *workspace_ptr);
@@ -104,6 +115,16 @@ static bool _wlmtk_root_element_keyboard_event(
 static void _wlmtk_root_handle_output_layout_change(
     struct wl_listener *listener_ptr,
     void *data_ptr);
+static void *_wlmtk_root_output_tracker_create(
+    struct wlr_output *wlr_output_ptr,
+    void *ud_ptr);
+static void _wlmtk_root_output_tracker_destroy(
+    struct wlr_output *wlr_output_ptr,
+    void *ud_ptr,
+    void *output_ptr);
+static void _wlmtk_root_output_handle_frame(
+    struct wl_listener *listener_ptr,
+    void *data_ptr);
 
 /** Virtual method table for the container's super class: Element. */
 static const wlmtk_element_vmt_t _wlmtk_root_element_vmt = {
@@ -123,6 +144,17 @@ wlmtk_root_t *wlmtk_root_create(
     if (NULL == root_ptr) return NULL;
     root_ptr->wlr_output_layout_ptr = wlr_output_layout_ptr;
 
+    root_ptr->output_tracker_ptr = wlmtk_output_tracker_create(
+        wlr_output_layout_ptr,
+        root_ptr,
+        _wlmtk_root_output_tracker_create,
+        NULL,
+        _wlmtk_root_output_tracker_destroy);
+    if (NULL == root_ptr->output_tracker_ptr) {
+        wlmtk_root_destroy(root_ptr);
+        return NULL;
+    }
+
     if (NULL != wlr_scene_ptr) {
         if (!wlmtk_container_init_attached(
                 &root_ptr->container,
@@ -189,6 +221,11 @@ void wlmtk_root_destroy(wlmtk_root_t *root_ptr)
 
     wlmtk_container_fini(&root_ptr->container);
 
+    if (NULL != root_ptr->output_tracker_ptr) {
+        wlmtk_output_tracker_destroy(root_ptr->output_tracker_ptr);
+        root_ptr->output_tracker_ptr = NULL;
+    }
+
     free(root_ptr);
 }
 
@@ -209,6 +246,7 @@ bool wlmtk_root_pointer_motion(
     root_ptr->mev = (wlmtk_pointer_motion_event_t){
         .x = x, .y = y, .time_msec = time_msec, .pointer_ptr = pointer_ptr
     };
+
     return wlmtk_element_pointer_motion(
         &root_ptr->container.super_element, &root_ptr->mev);
 }
@@ -672,6 +710,46 @@ void _wlmtk_root_handle_output_layout_change(
         root_ptr->extents.y);
 }
 
+/* ------------------------------------------------------------------------- */
+/** Ctor for struct wlmtk_root_output. */
+void *_wlmtk_root_output_tracker_create(
+    struct wlr_output *wlr_output_ptr,
+    void *ud_ptr)
+{
+    struct wlmtk_root_output *root_output_ptr = logged_calloc(
+        1, sizeof(struct wlmtk_root_output));
+    if (NULL == root_output_ptr) return NULL;
+    root_output_ptr->root_ptr = ud_ptr;
+    wlmtk_util_connect_listener_signal(
+        &wlr_output_ptr->events.frame,
+        &root_output_ptr->frame_listener,
+        _wlmtk_root_output_handle_frame);
+    return root_output_ptr;
+}
+
+/* ------------------------------------------------------------------------- */
+/** Dtor for struct wlmtk_root_output. */
+void _wlmtk_root_output_tracker_destroy(
+    __UNUSED__ struct wlr_output *wlr_output_ptr,
+    __UNUSED__ void *ud_ptr,
+    void *output_ptr)
+{
+    struct wlmtk_root_output *root_output_ptr = output_ptr;
+    wlmtk_util_disconnect_listener(&root_output_ptr->frame_listener);
+    free(root_output_ptr);
+}
+
+/* ------------------------------------------------------------------------- */
+/** Handles frame: Updates layout. */
+void _wlmtk_root_output_handle_frame(
+    struct wl_listener *listener_ptr,
+    __UNUSED__ void *data_ptr)
+{
+    struct wlmtk_root_output *root_output_ptr = BS_CONTAINER_OF(
+        listener_ptr, struct wlmtk_root_output, frame_listener);
+    wlmtk_element_layout(wlmtk_root_element(root_output_ptr->root_ptr));
+}
+
 /* == Unit tests =========================================================== */
 
 static void test_create_destroy(bs_test_t *test_ptr);
