@@ -29,7 +29,7 @@
 #include <wlr/types/wlr_keyboard.h>
 #include <wlr/types/wlr_scene.h>
 #include <wlr/types/wlr_seat.h>
-#include <wlr/version.h>
+#include <wlr/types/wlr_xdg_shell.h>
 #undef WLR_USE_UNSTABLE
 
 #include "element.h"
@@ -37,12 +37,56 @@
 #include "gfxbuf.h"  // IWYU pragma: keep
 #include "input.h"
 #include "util.h"
+#include "test.h"  // IWYU pragma: keep
 
 /* == Declarations ========================================================= */
 
+/** State of a `struct wlr_surface`, encapsuled for toolkit. */
+struct _wlmtk_surface_t {
+    /** Super class of the surface: An element. */
+    wlmtk_element_t           super_element;
+    /** Virtual method table of the super element before extending it. */
+    wlmtk_element_vmt_t       orig_super_element_vmt;
+    /** Seat. */
+    struct wlr_seat           *wlr_seat_ptr;
+
+    /** The `struct wlr_surface` wrapped. */
+    struct wlr_surface        *wlr_surface_ptr;
+    /** The `struct wlr_xdg_surface` wrapped. */
+    struct wlr_xdg_surface        *wlr_xdg_surface_ptr;
+
+    /** The scene API node displaying a surface and all it's sub-surfaces. */
+    struct wlr_scene_tree     *wlr_scene_tree_ptr;
+    /** Listener for the `destroy` signal of `wlr_scene_tree_ptr->node`. */
+    struct wl_listener        wlr_scene_tree_node_destroy_listener;
+
+    /** Committed width of the surface, in pixels. */
+    int                       committed_width;
+    /** Committed height of the surface, in pixels. */
+    int                       committed_height;
+
+    /** Listener for the `events.commit` signal of `wlr_surface`. */
+    struct wl_listener        surface_commit_listener;
+    /** Listener for the `map` signal of `wlr_surface`. */
+    struct wl_listener        surface_map_listener;
+    /** Listener for the `map` signal of `wlr_surface`. */
+    struct wl_listener        surface_unmap_listener;
+
+    /** Listener for @ref wlmtk_element_events_t::pointer_leave. */
+    struct wl_listener        element_pointer_leave_listener;
+    /** Listener for @ref wlmtk_element_events_t::pointer_motion. */
+    struct wl_listener        element_pointer_motion_listener;
+
+    /** Whether this surface is activated, ie. has keyboard focus. */
+    bool                      activated;
+};
+
 static bool _wlmtk_surface_init(
     wlmtk_surface_t *surface_ptr,
     struct wlr_surface *wlr_surface_ptr);
+static bool _wlmtk_xdg_surface_init(
+    wlmtk_surface_t *surface_ptr,
+    struct wlr_xdg_surface *wlr_xdg_surface_ptr);
 static void _wlmtk_surface_fini(wlmtk_surface_t *surface_ptr);
 
 static void _wlmtk_surface_element_destroy(wlmtk_element_t *element_ptr);
@@ -124,6 +168,23 @@ wlmtk_surface_t *wlmtk_surface_create(
     return surface_ptr;
 }
 
+/* ------------------------------------------------------------------------- */
+wlmtk_surface_t *wlmtk_xdg_surface_create(
+    struct wlr_xdg_surface *wlr_xdg_surface_ptr,
+    struct wlr_seat *wlr_seat_ptr)
+{
+    wlmtk_surface_t *surface_ptr = logged_calloc(1, sizeof(wlmtk_surface_t));
+    if (NULL == surface_ptr) return NULL;
+
+    if (!_wlmtk_xdg_surface_init(surface_ptr, wlr_xdg_surface_ptr)) {
+        wlmtk_surface_destroy(surface_ptr);
+        return NULL;
+    }
+    surface_ptr->wlr_seat_ptr = wlr_seat_ptr;
+
+    return surface_ptr;
+}
+
 /* ------------------------------------------------------------------------- */
 void wlmtk_surface_destroy(wlmtk_surface_t *surface_ptr)
 {
@@ -181,6 +242,12 @@ void wlmtk_surface_set_activated(
     surface_ptr->activated = activated;
 }
 
+/* ------------------------------------------------------------------------- */
+bool wlmtk_surface_is_activated(wlmtk_surface_t *surface_ptr)
+{
+    return surface_ptr->activated;
+}
+
 /* ------------------------------------------------------------------------- */
 void wlmtk_surface_connect_map_listener_signal(
     wlmtk_surface_t *surface_ptr,
@@ -255,7 +322,64 @@ bool _wlmtk_surface_init(
             &wlr_surface_ptr->events.unmap,
             &surface_ptr->surface_unmap_listener,
             _wlmtk_surface_handle_surface_unmap);
+        surface_ptr->wlr_surface_ptr->data = surface_ptr;
     }
+
+    return true;
+}
+
+/* ------------------------------------------------------------------------- */
+/**
+ * Initializes the surface, based off an XDG surface.
+ *
+ * TODO(kaeser@gubbe.ch): Merge with _wlmtk_surface_init.
+ *
+ * @param surface_ptr
+ * @param wlr_xdg_surface_ptr
+ *
+ * @return true on success.
+ */
+bool _wlmtk_xdg_surface_init(
+    wlmtk_surface_t *surface_ptr,
+    struct wlr_xdg_surface *wlr_xdg_surface_ptr)
+{
+    BS_ASSERT(NULL != surface_ptr);
+    *surface_ptr = (wlmtk_surface_t){};
+
+    if (!wlmtk_element_init(&surface_ptr->super_element)) {
+        _wlmtk_surface_fini(surface_ptr);
+        return false;
+    }
+    surface_ptr->orig_super_element_vmt = wlmtk_element_extend(
+        &surface_ptr->super_element, &surface_element_vmt);
+    wlmtk_util_connect_listener_signal(
+        &surface_ptr->super_element.events.pointer_leave,
+        &surface_ptr->element_pointer_leave_listener,
+        _wlmtk_surface_handle_element_pointer_leave);
+    wlmtk_util_connect_listener_signal(
+        &surface_ptr->super_element.events.pointer_motion,
+        &surface_ptr->element_pointer_motion_listener,
+        _wlmtk_surface_handle_element_pointer_motion);
+
+    surface_ptr->wlr_xdg_surface_ptr = wlr_xdg_surface_ptr;
+    surface_ptr->wlr_surface_ptr = wlr_xdg_surface_ptr->surface;
+    if (NULL != surface_ptr->wlr_surface_ptr) {
+        surface_ptr->wlr_surface_ptr->data = surface_ptr;
+        wlmtk_util_connect_listener_signal(
+            &surface_ptr->wlr_surface_ptr->events.commit,
+            &surface_ptr->surface_commit_listener,
+            _wlmtk_surface_handle_surface_commit);
+        wlmtk_util_connect_listener_signal(
+            &surface_ptr->wlr_surface_ptr->events.map,
+            &surface_ptr->surface_map_listener,
+            _wlmtk_surface_handle_surface_map);
+        wlmtk_util_connect_listener_signal(
+            &surface_ptr->wlr_surface_ptr->events.unmap,
+            &surface_ptr->surface_unmap_listener,
+            _wlmtk_surface_handle_surface_unmap);
+        surface_ptr->wlr_surface_ptr->data = surface_ptr;
+    }
+
     return true;
 }
 
@@ -273,6 +397,7 @@ void _wlmtk_surface_fini(wlmtk_surface_t *surface_ptr)
     }
 
     if (NULL != surface_ptr->wlr_surface_ptr) {
+        surface_ptr->wlr_surface_ptr->data = NULL;
         surface_ptr->wlr_surface_ptr = NULL;
         wlmtk_util_disconnect_listener(&surface_ptr->surface_commit_listener);
         wlmtk_util_disconnect_listener(&surface_ptr->surface_map_listener);
@@ -314,8 +439,13 @@ struct wlr_scene_node *_wlmtk_surface_element_create_scene_node(
         element_ptr, wlmtk_surface_t, super_element);
     BS_ASSERT(NULL == surface_ptr->wlr_scene_tree_ptr);
 
-    surface_ptr->wlr_scene_tree_ptr = wlr_scene_subsurface_tree_create(
-        wlr_scene_tree_ptr, surface_ptr->wlr_surface_ptr);
+    if (NULL != surface_ptr->wlr_xdg_surface_ptr) {
+        surface_ptr->wlr_scene_tree_ptr = wlr_scene_xdg_surface_create(
+            wlr_scene_tree_ptr, surface_ptr->wlr_xdg_surface_ptr);
+    } else {
+        surface_ptr->wlr_scene_tree_ptr = wlr_scene_subsurface_tree_create(
+            wlr_scene_tree_ptr, surface_ptr->wlr_surface_ptr);
+    }
     if (NULL == surface_ptr->wlr_scene_tree_ptr) return NULL;
 
     wlmtk_util_connect_listener_signal(
@@ -426,15 +556,9 @@ bool _wlmtk_surface_element_pointer_button(
     // We're only forwarding PRESSED & RELEASED events.
     if (WLMTK_BUTTON_DOWN == button_event_ptr->type ||
         WLMTK_BUTTON_UP == button_event_ptr->type) {
-#if WLR_VERSION_NUM >= (18 << 8)
         enum wl_pointer_button_state state =
             (button_event_ptr->type == WLMTK_BUTTON_DOWN) ?
             WL_POINTER_BUTTON_STATE_PRESSED : WL_POINTER_BUTTON_STATE_RELEASED;
-#else // WLR_VERSION_NUM >= (18 << 8)
-        enum wlr_button_state state =
-            (button_event_ptr->type == WLMTK_BUTTON_DOWN) ?
-            WLR_BUTTON_PRESSED : WLR_BUTTON_RELEASED;
-#endif // WLR_VERSION_NUM >= (18 << 8)
         wlr_seat_pointer_notify_button(
             surface_ptr->wlr_seat_ptr,
             button_event_ptr->time_msec,
@@ -475,9 +599,7 @@ bool _wlmtk_surface_element_pointer_axis(
         wlr_pointer_axis_event_ptr->delta,
         wlr_pointer_axis_event_ptr->delta_discrete,
         wlr_pointer_axis_event_ptr->source
-#if WLR_VERSION_NUM >= (18 << 8)
         , wlr_pointer_axis_event_ptr->relative_direction
-#endif // WLR_VERSION_NUM >= (18 << 8)
         );
     return true;
 }
@@ -688,136 +810,20 @@ void _wlmtk_surface_commit_size(
     }
 
     if (NULL != surface_ptr->super_element.parent_container_ptr) {
-        wlmtk_container_update_layout_and_pointer_focus(
+        wlmtk_container_invalidate_layout(
             surface_ptr->super_element.parent_container_ptr);
     }
 }
 
 
-/* == Fake surface methods ================================================= */
-
-static void _wlmtk_fake_surface_element_destroy(
-    wlmtk_element_t *element_ptr);
-static struct wlr_scene_node *_wlmtk_fake_surface_element_create_scene_node(
-    wlmtk_element_t *element_ptr,
-    struct wlr_scene_tree *wlr_scene_tree_ptr);
-static bool _wlmtk_fake_surface_element_pointer_accepts_motion(
-    wlmtk_element_t *element_ptr,
-    wlmtk_pointer_motion_event_t *motion_event_ptr);
-static bool _wlmtk_fake_surface_element_pointer_button(
-    wlmtk_element_t *element_ptr,
-    const wlmtk_button_event_t *button_event_ptr);
-
-/** Extensions to the surface's super elements virtual methods. */
-static const wlmtk_element_vmt_t _wlmtk_fake_surface_element_vmt = {
-    .destroy = _wlmtk_fake_surface_element_destroy,
-    .create_scene_node = _wlmtk_fake_surface_element_create_scene_node,
-    .pointer_accepts_motion = _wlmtk_fake_surface_element_pointer_accepts_motion,
-    .pointer_button = _wlmtk_fake_surface_element_pointer_button,
-};
-
-/* ------------------------------------------------------------------------- */
-wlmtk_fake_surface_t *wlmtk_fake_surface_create(void)
-{
-    wlmtk_fake_surface_t *fake_surface_ptr = logged_calloc(
-        1, sizeof(wlmtk_fake_surface_t));
-    if (NULL == fake_surface_ptr) return NULL;
-
-    _wlmtk_surface_init(&fake_surface_ptr->surface, NULL);
-    wlmtk_element_extend(
-        &fake_surface_ptr->surface.super_element,
-        &_wlmtk_fake_surface_element_vmt);
-    return fake_surface_ptr;
-}
-
-/* ------------------------------------------------------------------------- */
-wlmtk_surface_t *wlmtk_fake_surface_create_inject(
-    __UNUSED__ struct wlr_surface *wlr_surface_ptr,
-    __UNUSED__ struct wlr_seat *wlr_seat_ptr)
-{
-    wlmtk_fake_surface_t *fake_surface_ptr = wlmtk_fake_surface_create();
-    if (NULL == fake_surface_ptr) return NULL;
-    return &fake_surface_ptr->surface;
-}
-
-/* ------------------------------------------------------------------------- */
-void wlmtk_fake_surface_commit_size(
-    wlmtk_fake_surface_t *fake_surface_ptr,
-    int width,
-    int height)
-{
-    _wlmtk_surface_commit_size(&fake_surface_ptr->surface, width, height);
-}
-
-/* ------------------------------------------------------------------------- */
-void wlmtk_fake_surface_destroy(wlmtk_fake_surface_t *fake_surface_ptr)
-{
-    _wlmtk_surface_fini(&fake_surface_ptr->surface);
-    free(fake_surface_ptr);
-}
-
-/* ------------------------------------------------------------------------- */
-/** Fake implementation of the dtor, @ref wlmtk_element_vmt_t::destroy. */
-void _wlmtk_fake_surface_element_destroy(
-    wlmtk_element_t *element_ptr)
-{
-    wlmtk_fake_surface_t *fake_surface_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_fake_surface_t, surface.super_element);
-    wlmtk_fake_surface_destroy(fake_surface_ptr);
-}
-
-/* ------------------------------------------------------------------------- */
-/** Fake implementation of @ref wlmtk_element_vmt_t::create_scene_node. */
-struct wlr_scene_node *_wlmtk_fake_surface_element_create_scene_node(
-    wlmtk_element_t *element_ptr,
-    struct wlr_scene_tree *wlr_scene_tree_ptr)
-{
-    wlmtk_fake_surface_t *fake_surface_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_fake_surface_t, surface.super_element);
-
-    struct wlr_buffer *wlr_buffer_ptr = bs_gfxbuf_create_wlr_buffer(
-        fake_surface_ptr->surface.committed_width,
-        fake_surface_ptr->surface.committed_height);
-    BS_ASSERT(NULL != wlr_buffer_ptr);
-
-    struct wlr_scene_buffer *wlr_scene_buffer_ptr = wlr_scene_buffer_create(
-        wlr_scene_tree_ptr, wlr_buffer_ptr);
-    wlr_buffer_drop(wlr_buffer_ptr);
-   return &wlr_scene_buffer_ptr->node;
-}
-
-/* ------------------------------------------------------------------------- */
-/** Fake for @ref wlmtk_element_vmt_t::pointer_accepts_motion. True if in. */
-bool _wlmtk_fake_surface_element_pointer_accepts_motion(
-    wlmtk_element_t *element_ptr,
-    wlmtk_pointer_motion_event_t *motion_event_ptr)
-{
-    wlmtk_fake_surface_t *fake_surface_ptr = BS_CONTAINER_OF(
-        element_ptr, wlmtk_fake_surface_t, surface.super_element);
-
-    return (0 <= motion_event_ptr->x &&
-            motion_event_ptr->x < fake_surface_ptr->surface.committed_width &&
-            0 <= motion_event_ptr->y &&
-            motion_event_ptr->y < fake_surface_ptr->surface.committed_height);
-}
-
-/* ------------------------------------------------------------------------- */
-/** Fake for @ref wlmtk_element_vmt_t::pointer_button. Returns true. */
-bool _wlmtk_fake_surface_element_pointer_button(
-    __UNUSED__ wlmtk_element_t *element_ptr,
-    __UNUSED__ const wlmtk_button_event_t *button_event_ptr)
-{
-    return true;
-}
-
 /* == Unit tests =========================================================== */
 
 static void test_create_destroy(bs_test_t *test_ptr);
-static void test_fake_commit(bs_test_t *test_ptr);
+static void test_dimensions(bs_test_t *test_ptr);
 
 const bs_test_case_t wlmtk_surface_test_cases[] = {
     { 1, "create_destroy", test_create_destroy },
-    { 1, "fake_commit", test_fake_commit },
+    { 1, "dimensions", test_dimensions },
     { 0, NULL, NULL }
 };
 
@@ -837,22 +843,32 @@ void test_create_destroy(bs_test_t *test_ptr)
 }
 
 /* ------------------------------------------------------------------------- */
-/** Exercises the request_size / commit flow. */
-void test_fake_commit(bs_test_t *test_ptr)
+/** Verifies that dimensions from the surface propagate thorugh. */
+void test_dimensions(bs_test_t *test_ptr)
 {
-    wlmtk_fake_surface_t *fake_surface_ptr = wlmtk_fake_surface_create();
-    int w, h;
+    struct wlr_surface wlr_surface = {};
+    wl_list_init(&wlr_surface.current.subsurfaces_below);
+    wl_list_init(&wlr_surface.current.subsurfaces_above);
+    wl_signal_init(&wlr_surface.events.commit);
+    wl_signal_init(&wlr_surface.events.destroy);
+    wl_signal_init(&wlr_surface.events.map);
+    wl_signal_init(&wlr_surface.events.unmap);
 
-    wlmtk_surface_get_size(&fake_surface_ptr->surface, &w, &h);
-    BS_TEST_VERIFY_EQ(test_ptr, 0, w);
-    BS_TEST_VERIFY_EQ(test_ptr, 0, h);
+    wlmtk_surface_t *s = wlmtk_surface_create(&wlr_surface, NULL);
+    BS_TEST_VERIFY_NEQ(test_ptr, NULL, s);
 
-    wlmtk_fake_surface_commit_size(fake_surface_ptr, 200, 100);
-    wlmtk_surface_get_size(&fake_surface_ptr->surface, &w, &h);
-    BS_TEST_VERIFY_EQ(test_ptr, 200, w);
-    BS_TEST_VERIFY_EQ(test_ptr, 100, h);
+    WLMTK_TEST_VERIFY_WLRBOX_EQ(
+        test_ptr, 0, 0, 0, 0,
+        wlmtk_element_get_dimensions_box(wlmtk_surface_element(s)));
 
-    wlmtk_fake_surface_destroy(fake_surface_ptr);
+    wlr_surface.current.width = 640;
+    wlr_surface.current.height = 480;
+    wl_signal_emit(&wlr_surface.events.commit, NULL);
+    WLMTK_TEST_VERIFY_WLRBOX_EQ(
+        test_ptr, 0, 0, 640, 480,
+        wlmtk_element_get_dimensions_box(wlmtk_surface_element(s)));
+
+    wlmtk_surface_destroy(s);
 }
 
 /* == End of surface.c ===================================================== */
