@@ -20,19 +20,19 @@
 
 #include "titlebar.h"
 
-#include "box.h"
-#include "button.h"
-#include "buffer.h"
-#include "gfxbuf.h"
-#include "primitives.h"
-#include "titlebar_button.h"
-#include "titlebar_title.h"
-#include "window.h"
-
+#include <cairo.h>
+#include <libbase/libbase.h>
+#include <stdlib.h>
 #define WLR_USE_UNSTABLE
 #include <wlr/interfaces/wlr_buffer.h>
 #undef WLR_USE_UNSTABLE
 
+#include "box.h"
+#include "container.h"
+#include "primitives.h"
+#include "titlebar_button.h"
+#include "titlebar_title.h"
+
 /* == Declarations ========================================================= */
 
 /** State of the title bar. */
@@ -95,18 +95,17 @@ static const uint32_t _wlmtk_titlebar_default_properties =
 /* == Exported methods ===================================================== */
 
 /* ------------------------------------------------------------------------- */
-wlmtk_titlebar_t *wlmtk_titlebar_create(
-    wlmtk_env_t *env_ptr,
+wlmtk_titlebar_t *wlmtk_titlebar2_create(
     wlmtk_window_t *window_ptr,
     const wlmtk_titlebar_style_t *style_ptr)
 {
     wlmtk_titlebar_t *titlebar_ptr = logged_calloc(
         1, sizeof(wlmtk_titlebar_t));
     if (NULL == titlebar_ptr) return NULL;
-    memcpy(&titlebar_ptr->style, style_ptr, sizeof(wlmtk_titlebar_style_t));
+    titlebar_ptr->style = *style_ptr;
     titlebar_ptr->title_ptr = wlmtk_window_get_title(window_ptr);
 
-    if (!wlmtk_box_init(&titlebar_ptr->super_box, env_ptr,
+    if (!wlmtk_box_init(&titlebar_ptr->super_box,
                         WLMTK_BOX_HORIZONTAL,
                         &titlebar_ptr->style.margin)) {
         wlmtk_titlebar_destroy(titlebar_ptr);
@@ -116,8 +115,7 @@ wlmtk_titlebar_t *wlmtk_titlebar_create(
         &titlebar_ptr->super_box.super_container.super_element,
         &titlebar_element_vmt);
 
-    titlebar_ptr->titlebar_title_ptr = wlmtk_titlebar_title_create(
-        env_ptr, window_ptr);
+    titlebar_ptr->titlebar_title_ptr = wlmtk_titlebar2_title_create(window_ptr);
     if (NULL == titlebar_ptr->titlebar_title_ptr) {
         wlmtk_titlebar_destroy(titlebar_ptr);
         return NULL;
@@ -126,8 +124,7 @@ wlmtk_titlebar_t *wlmtk_titlebar_create(
         &titlebar_ptr->super_box,
         wlmtk_titlebar_title_element(titlebar_ptr->titlebar_title_ptr));
 
-    titlebar_ptr->minimize_button_ptr = wlmtk_titlebar_button_create(
-        env_ptr,
+    titlebar_ptr->minimize_button_ptr = wlmtk_titlebar2_button_create(
         wlmtk_window_request_minimize,
         window_ptr,
         wlmaker_primitives_draw_minimize_icon);
@@ -139,8 +136,7 @@ wlmtk_titlebar_t *wlmtk_titlebar_create(
         &titlebar_ptr->super_box,
         wlmtk_titlebar_button_element(titlebar_ptr->minimize_button_ptr));
 
-    titlebar_ptr->close_button_ptr = wlmtk_titlebar_button_create(
-        env_ptr,
+    titlebar_ptr->close_button_ptr = wlmtk_titlebar2_button_create(
         wlmtk_window_request_close,
         window_ptr,
         wlmaker_primitives_draw_close_icon);
@@ -212,7 +208,9 @@ bool wlmtk_titlebar_set_width(
     if (!redraw(titlebar_ptr)) return false;
 
     // Don't forget to re-position the elements.
-    wlmtk_container_update_layout(&titlebar_ptr->super_box.super_container);
+    wlmtk_element_layout(wlmtk_box_element(&titlebar_ptr->super_box));
+    wlmtk_container_invalidate_layout(
+        &titlebar_ptr->super_box.super_container);
     return true;
 }
 
@@ -228,7 +226,9 @@ void wlmtk_titlebar_set_properties(
     if (!redraw(titlebar_ptr)) return;
 
     // Don't forget to re-position the elements.
-    wlmtk_container_update_layout(&titlebar_ptr->super_box.super_container);
+    wlmtk_element_layout(wlmtk_box_element(&titlebar_ptr->super_box));
+    wlmtk_container_invalidate_layout(
+        &titlebar_ptr->super_box.super_container);
 }
 
 /* ------------------------------------------------------------------------- */
@@ -424,40 +424,24 @@ bool redraw(wlmtk_titlebar_t *titlebar_ptr)
 
 /* == Unit tests =========================================================== */
 
-static void test_create_destroy(bs_test_t *test_ptr);
 static void test_variable_width(bs_test_t *test_ptr);
 static void test_properties(bs_test_t *test_ptr);
 
 const bs_test_case_t wlmtk_titlebar_test_cases[] = {
-    { 1, "create_destroy", test_create_destroy },
     { 1, "variable_width", test_variable_width },
     { 1, "properties", test_properties },
     { 0, NULL, NULL }
 };
 
-/* ------------------------------------------------------------------------- */
-/** Tests setup and teardown. */
-void test_create_destroy(bs_test_t *test_ptr)
-{
-    wlmtk_fake_window_t *fake_window_ptr = wlmtk_fake_window_create();
-    wlmtk_titlebar_style_t style = {};
-    wlmtk_titlebar_t *titlebar_ptr = wlmtk_titlebar_create(
-        NULL, fake_window_ptr->window_ptr, &style);
-    BS_TEST_VERIFY_NEQ(test_ptr, NULL, titlebar_ptr);
-
-    wlmtk_element_destroy(wlmtk_titlebar_element(titlebar_ptr));
-    wlmtk_fake_window_destroy(fake_window_ptr);
-}
-
 /* ------------------------------------------------------------------------- */
 /** Tests titlebar with variable width. */
 void test_variable_width(bs_test_t *test_ptr)
 {
-    wlmtk_fake_window_t *fake_window_ptr = wlmtk_fake_window_create();
+    wlmtk_window_t *w = wlmtk_test_window_create(NULL);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, w);
     wlmtk_titlebar_style_t style = { .height = 22, .margin = { .width = 2 } };
-    wlmtk_titlebar_t *titlebar_ptr = wlmtk_titlebar_create(
-        NULL, fake_window_ptr->window_ptr, &style);
-    BS_TEST_VERIFY_NEQ(test_ptr, NULL, titlebar_ptr);
+    wlmtk_titlebar_t *titlebar_ptr = wlmtk_titlebar2_create(w, &style);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, titlebar_ptr);
 
     // Short names, for improved readability.
     wlmtk_element_t *title_elem_ptr = wlmtk_titlebar_title_element(
@@ -502,18 +486,18 @@ void test_variable_width(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(test_ptr, 66, width);
 
     wlmtk_element_destroy(wlmtk_titlebar_element(titlebar_ptr));
-    wlmtk_fake_window_destroy(fake_window_ptr);
+    wlmtk_window_destroy(w);
 }
 
 /* ------------------------------------------------------------------------- */
 /** Tests titlebar with configured properties. */
 void test_properties(bs_test_t *test_ptr)
 {
-    wlmtk_fake_window_t *fake_window_ptr = wlmtk_fake_window_create();
+    wlmtk_window_t *w = wlmtk_test_window_create(NULL);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, w);
     wlmtk_titlebar_style_t style = { .height = 22, .margin = { .width = 2 } };
-    wlmtk_titlebar_t *titlebar_ptr = wlmtk_titlebar_create(
-        NULL, fake_window_ptr->window_ptr, &style);
-    BS_TEST_VERIFY_NEQ(test_ptr, NULL, titlebar_ptr);
+    wlmtk_titlebar_t *titlebar_ptr = wlmtk_titlebar2_create(w, &style);
+    BS_TEST_VERIFY_NEQ_OR_RETURN(test_ptr, NULL, titlebar_ptr);
 
     // Short names, for improved readability.
     wlmtk_element_t *title_elem_ptr = wlmtk_titlebar_title_element(
@@ -575,7 +559,7 @@ void test_properties(bs_test_t *test_ptr)
     BS_TEST_VERIFY_EQ(test_ptr, 67, close_elem_ptr->x);
 
     wlmtk_element_destroy(wlmtk_titlebar_element(titlebar_ptr));
-    wlmtk_fake_window_destroy(fake_window_ptr);
+    wlmtk_window_destroy(w);
 }
 
 /* == End of titlebar.c ==================================================== */
