@@ -20,25 +20,42 @@
 #ifndef __XDG_TOPLEVEL_H__
 #define __XDG_TOPLEVEL_H__
 
+#include <libbase/libbase.h>
+#include <stdbool.h>
+
 #include "server.h"
-#include "toolkit/toolkit.h"
+
+struct wlr_xdg_toplevel;
+struct wlmaker_xdg_toplevel;
 
 #ifdef __cplusplus
 extern "C" {
 #endif  // __cplusplus
 
 /**
- * Creates a toolkit window with the XDG surface as content.
+ * Creates an XDG toplevel.
  *
  * @param wlr_xdg_toplevel_ptr
  * @param server_ptr
  *
- * @return The window, or NULL on error.
+ * @return wlmaker's toplevel handle or NULL on error.
  */
-wlmtk_window_t *wlmtk_window_create_from_xdg_toplevel(
+struct wlmaker_xdg_toplevel *wlmaker_xdg_toplevel_create(
     struct wlr_xdg_toplevel *wlr_xdg_toplevel_ptr,
     wlmaker_server_t *server_ptr);
 
+/** Dtor for the XDG toplevel. */
+void wlmaker_xdg_toplevel_destroy(
+    struct wlmaker_xdg_toplevel *wlmaker_xdg_toplevel_ptr);
+
+/** En-/Disables server-side decoration for the XDG toplevel. */
+void wlmaker_xdg_toplevel_set_server_side_decorated(
+    struct wlmaker_xdg_toplevel *wlmaker_xdg_toplevel_ptr,
+    bool server_side_decorated);
+
+/** Unit test set. */
+extern const bs_test_set_t wlmaker_xdg_toplevel_test_set;
+
 #ifdef __cplusplus
 }  // extern "C"
 #endif  // __cplusplus
